      subroutine ehenx(ei,q2,q0,ipol,thsa,phisa,ktf,pfx,pfy,pfz,
     &               crspl,crsmn,crs,ith3,icon,eps0)
*******************************************************************************************
*      This subroutine calculates the 3He(e,e'N)X cross section in 
*      dsigma/dEe d\omega_e dp_f d\omega_f  nb/GeV^2/Sr^2
*
*  ei - initial electron energy (GeV)
*  q2 - Q2  (GeV^2)
*  q0 - virtual photon energy (GeV)
*  ipol=0 - no polarizataion is considered
*  ipol=1 - polarization observables are  considered
*  thsa  - target polarization vector's polar angle vs q (rad)
*  phisa - target polarization vector's azimuthal angle vs (qe) (rad)
*  ktf   - type of knoced-out nucleon -1 (neutron), 1 (proton)
*  pfx,pfy,pfz components of knocked-out nucleon momentum (GeV)

*  crspl  - cross section for + helicity electron  (output)
*  crsmn  - cross section for - helicity electron  (output)
*  crs    - unpolorized cross secton
*  ith3   - 1 - 3He, -1 - 3H nucleus
*
* icon = -1 initalizes, 1-PWIA, 13-(PWIA+SingleFSI)
*           14-(PWIA+Single Chex), 134 (PWIA+ single FSI + single Chex)
*         
* eps0 - accuracy of integration - 0.1 - 10%
*
*   Coordinate system is defined such that z||q and zx is (q e) e initial electron momentum
*
*
* comment:  Unpolorized cross section can be calculated in two ways:
* ipol=1  then considering (crspl+crsmn)/2.0
* ipol=0  considering only crs  - this option will work faster
* note that in case of ipol=0, one can choose any value for thsa and phisa
*************************************************************************************************
      external under_ehen,phir2_a,phir2_b
      common/par/pi,pm,pmp,pmn,tm,eb
      common/proton_num/z	
      common/atomic_num/an

      common/prointr2/eei,qq2,qq0,qv,tthsa,pphisa,kktf,ppfx,ppfy,ppfz,
     &                iic
      common/cross_case/icase
      real*8 ei_d,q2_d,q0_d,thsa_d,phisa_d
      real*8 pfx_d,pfy_d,pfz_d
      real*8 pr2_d,thr2_d,phir2_d
      real*8 sa
      real*8 crspl_d,crsmn_d


      crspl = 0.0
      crsmn = 0.0


*********************************************************************
*  Initialization
*********************************************************************      
      if(icon.eq.-1)then   ! initialization
        pm  = 0.938279
        pmp = pm
        pmn = 0.939565  
        an  = 3.0

        if(ith3.eq.1)then ! he3
	z   = 2.0
	eb  = 0.007718109 
        elseif(ith3.eq.-1)then  ! h3
	z   = 1.0
	eb  = 0.0085
        endif

	am  = z*pm + (an-z)*pmn  - eb   ! target mass
        tm  = am                        ! again target mass

*      icon = -1                  !initialize
      call eheppn_in(ei_d,q2_d,q0_d,sa,thsa_d,phisa_d,ktf,pfx_d,pfy_d,
     &       pfz_d,ktr2,pr2_d,thr2_d,phir2_d,crspl_d,crsmn_d,icon,ifln)
      return
      endif


*************************************************************************
*  Calculations
*************************************************************************
      eei = ei
      qq2 = q2
      qq0 = q0
      qv = sqrt(q2+q0**2)
      tthsa  = thsa
      pphisa = phisa
      kktf   = ktf
      ppfx   = pfx
      ppfy   = pfy
      ppfz   = pfz
      iic    = icon
      
      if(ipol.eq.1)then  
      icase = 1               ! case of + helicity of electron
      thr2_a = 0.0
      thr2_b = pi
      eps = eps0 !0.1
      call  gadaps2(thr2_a,thr2_b,phir2_a,phir2_b,under_ehen,eps,sum)  
      crspl = sum

      icase = 2                ! case of - helicity of electron
      thr2_a = 0.0
      thr2_b = pi
      eps = eps0 !0.1
      call  gadaps2(thr2_a,thr2_b,phir2_a,phir2_b,under_ehen,eps,sum)  
      crsmn = sum

      elseif(ipol.eq.0)then  
      icase = 0                ! unpolorized case
      thr2_a = 0.0
      thr2_b = pi
      eps = eps0 !0.1
      call  gadaps2(thr2_a,thr2_b,phir2_a,phir2_b,under_ehen,eps,sum)  
      crs = sum
      endif

      return
      end
      

      function phir2_a(thr2)
      phir2_a = 0.0
      return
      end

      function phir2_b(thr2)
      phir2_b = 2.0*acos(-1.0)
      return
      end




************************************************************************
*    Ingefrant of \int under_ehen d(thr2) d(phir2)
************************************************************************
      function under_ehen(thr2,phir2)
      common/proton_num/z
      real*8 ei_d,q2_d,q0_d,thsa_d,phisa_d
      real*8 pfx_d,pfy_d,pfz_d
      real*8 pr2_d,thr2_d,phir2_d
      real*8 sa
      real*8 crspl_d,crsmn_d
      dimension pr2sol(2),er2sol(2)
      common/prointr2/ei,q2,q0,qv,thsa,phisa,ktf,pfx,pfy,pfz,icon
      common/cross_case/icase
***********************************************************
      crspl = 0.0
      crsmn = 0.0
      crs   = 0.0

      pf = sqrt(pfx**2 + pfy**2 + pfz**2)
      call polar(pfx,pfy,pfz,pf,thf,phif)

      call kin_get_pr2(q2,qv,q0,pf,thf,phif,thr2,phir2,pr2sol,ifl)
      if(ifl.eq.0)goto 1

      do i = 1,ifl

      pr2  =    pr2sol(i)
   
      ei_d    = dble(ei)
      q2_d    = dble(q2)
      q0_d    = dble(q0)
      thsa_d  = dble(thsa)
      phisa_d = dble(phisa)
      pfx_d   = dble(pfx)
      pfy_d   = dble(pfy)
      pfz_d   = dble(pfz)
      pr2_d   = dble(pr2)
      thr2_d  = dble(thr2)
      phir2_d = dble(phir2)
      sa      = 1.0

      do ktr2 = -1,1,2
       
*         print *,"z",z
       if(z.gt.1.5)then ! He3
       ktr3 = 1 - (ktf+ktr2)
*      if(ktf.eq.1.and.ktr2.eq.-1)ktr3 = 1
*      if(ktf.eq.1.and.ktr2.eq.1) ktr3 = -1
*      if(ktf.eq.-1.and.ktr2.eq.-1)goto 2 
*      if(ktf.eq.-1.and.ktr2.eq.1) ktr3 = 1
       elseif(z.lt.1.5)then ! H3
       ktr3 = -1 - (ktf+ktr2) 
*      if(ktf.eq.1.and.ktr2.eq.-1)ktr3 = -1
*      if(ktf.eq.1.and.ktr2.eq.1) goto 2
*      if(ktf.eq.-1.and.ktr2.eq.-1)ktr3 = 1 
*      if(ktf.eq.-1.and.ktr2.eq.1) ktr3 = -1
*       print *,"***",ktf,ktr2,ktr3
      endif

      if(abs(ktr3).ne.1)goto 2

      call eheppn_in(ei_d,q2_d,q0_d,sa,thsa_d,phisa_d,ktf,pfx_d,pfy_d,
     &       pfz_d,ktr2,pr2_d,thr2_d,phir2_d,crspl_d,crsmn_d,icon,ifln)


      crspl = crspl + sngl(crspl_d)*sin(thr2)
      crsmn = crsmn + sngl(crsmn_d)*sin(thr2)
 2    continue
      enddo
      enddo

 1    continue      
      if(icase.eq.0)under_ehen = (crspl+crsmn)/2.0
      if(icase.eq.1)under_ehen = crspl
      if(icase.eq.2)under_ehen = crsmn

      return
      end
 
*     *******************************************************************
      subroutine kin_get_pr2(q2,qv,q0,pf,thf,phif,thr2,phir2,pr2sol,ifl)
      common/par/pi,pm,pmp,pmn,tm,eb
      dimension pr2sol(2)
      ifl = 0
      pr2sol(1)=0.0
      pr2sol(2)=0.0

      ef = sqrt(pm**2 + pf**2)
      cs_th_r2f = cos(thr2)*cos(thf)+sin(thr2)*sin(thf)*cos(phir2-phif)

      arm_a = -q2 + 2.0*q0*tm + tm**2 - 2.0*ef*(q0+tm) + 
     &         2.0*pf*qv*cos(thf) + pm**2
      arm_b = 2.0*(qv*cos(thr2) - pf*cs_th_r2f)
      arm_g = 2.0*(q0+tm-ef)

      A = arm_b**2 - arm_g**2
      B = 2.0*arm_a*arm_b
      C = arm_a**2 - pm**2*arm_g**2
*      print *,"arm",arm_a,arm_b,arm_g,cs_th_r2f
*      print *,"ABC",A,B,C,B**2 - 4.*A*C,ef
	if(A.eq.0.0)then
	pr2sol(1) = -C/B
        ifl = 1
        return

 	else

   	call  squar_eq(a,b,c,x1,x2,i_test)
	if(i_test.eq.1)return
*	print *,'xx',x1,x2
	test1  = (arm_a + x1*arm_b)/arm_g  
	test2  = (arm_a + x2*arm_b)/arm_g  

        if(x1.lt.0.0.and.x2.lt.0.0)return
        
        ifl=1
	pr2sol(1)  = x1		
	if(test1.lt.pm.or.x1.lt.0.0)then
 	if(test2.lt.pm.or.x2.lt.0.0)return
	pr2sol(1)  = x2

	elseif(test2.gt.pm.and.x2.gt.0.0)then
        pr2sol(2)  = x2
        ifl = 2
	endif

	endif

 	return
 	end

*     *******************************************************************
      subroutine kin_get_er2(q2,qv,q0,pf,thf,phif,thr2,phir2,er2sol,ifl)
      common/par/pi,pm,pmp,pmn,tm,eb
      dimension er2sol(2)
      ifl = 0
      er2sol(1)=0.0
      er2sol(2)=0.0

      ef = sqrt(pm**2 + pf**2)
      cs_th_r2f = cos(thr2)*cos(thf)+sin(thr2)*sin(thf)*cos(phir2-phif)

      arm_a = (q0+tm-ef)**2
      arm_g = 2.0*(q0+tm-ef)

      A = 1.0
      B = -arm_g 
      C = arm_a**2 - pm**2
*      print *,"arm",arm_a,arm_b,arm_g,cs_th_r2f
*      print *,"ABC",A,B,C,B**2 - 4.*A*C,ef
   	call  squar_eq(a,b,c,x1,x2,i_test)
	if(i_test.eq.1)return
*	print *,'xx',x1,x2
	test1  = x1**2 - pm**2
	test2  = x2**2 - pm**2

        if(x1.lt.0.0.and.x2.lt.0.0)return
        
        ifl=1
	er2sol(1)  = x1		
	if(test1.lt.0.0.or.x1.lt.0.0)then
 	if(test2.lt.0.0.or.x2.lt.0.0)return
	er2sol(1)  = x2

	elseif(test2.gt.0.0.and.x2.gt.0.0)then
        er2sol(2)  = x2
        ifl = 2
	endif

 	return
 	end




*     ******************************************************
      subroutine nucleon_asym(kt,q2,ue,thsa,phisa,asym)
************************************************************
*   Asymmetry for the case of isolated nucleon at rest
*  kt = 1 - proton
*  kt = -1 neutronn
*  q2 = Q2
*  thsa  = polar angle of polarization vector
*  phisa = azimuthal angle of polarization vector
*  with respect to q
*
************************************************************
       common/par/pi,pm,pmp,pmn,tm,eb
        tau = q2/(4.0*pm**2)
        tn  = tan(ue/2.0)
        ics = 2
        if(kt.eq.-1)then
        call neutron_formfactors(q2,GE,GM,ics)
        else
        call  proton_formfactors(q2,GE,GM,ics)
        endif
         
         pol1 = sqrt(tau*(1.0+(1.0+tau)*tn**2))*GM**2*cos(thsa)
         pol2 = GM*GE*sin(thsa)*cos(phisa)
         delt = -2.0*tn*sqrt(tau/(1.0+tau))*(pol1+pol2)

         sigm = (GE**2+tau*GM**2)/(1.0+tau) + 2.0*tau*tn**2*GM**2

         asym = delt/sigm
         return
         end





******************************************************************
* all subroutines below belong to eheppn_in
******************************************************************

       subroutine eheppn_in(ei,q2,q0,sa,thsa,phisa,ktf,pfx,pfy,pfz,
     &                 ktr2,pr2,thr2,phir2,crspl,crsmn,icon,ifl)
**********************************************************************
* eheppn.f - v.3.50
*
* Cross section of He3(e,e'ppn) scattering 
* Ei   - initial electron energy (GeV)                     - input

* s2,s1 - 3d polarization vectors for scattered and 
*        incoming electrons                                - nused
* hei 1,-1 helicity of incoming electron                   - nused

* Q2   - Q2 GeV^2                                          - input
* q0   - energy of virtual nucleon                         - input
*
* sa   - polarization magnitude (usually taken as 1)       - input
* thsa - polar angle of He3 spin vs q                      - input
* phisa - azimuthal angle of He3 vs qx                     - input

* ktf  - type of knocked-out nucleon -1 neutreon 1 proton  - input
* pfx  - x momentum of knocked-out nucleon                 - input
* pfy  - y momentum of knocked-out nucleon                 - input
* ktr2 - registered recoil nucleon -1 neutreon 1 proton    - input
* pr2  - momentum of registered spectator nucleon          - input
* thr2 - polar angle (in radian) of spectator nucleon      - input
* phir2- azimuthal angle (in radian) of spectator nucleon  - input
* pfz  - longitudinal   momentum of knocked - out nucleon  - input
* crs  - cross section                                     - output
* crspl - cross section for + helicity electron            - output
* crsmn - cross section for - helicity electron            - output
* icon - 1 - pwia, 12-pwia+pair dist, 3, FSI, 13 - pwia+FSI1
*        5 - Charge excgange,  6 - nondiagonal FSI         - input
* icon - -1 - initialize the subroutine,                   - input
*        1 - calculate the cross sections as 
*            dsigma/dEe,domega_e, dp_f, domega_f, domega_r 
*        2 - calculate the cross sections as 
*            dsigma/dEe,domega_e, dp_r, domega_r, domega_f 
*         (r - subscript means recoil/spectator nucleon          
* ifl - 0 tells that it is kinematically not forbidden, 1-forbidden 
*
*
* The azimuthal angles are defined in the reference frame where 
* q || z and xz is the (eq), e- is the initial electron
*
* cross sections is  in nb/GeV^2/str^3
*
*
*
* Misak Sargsian
* FIU, March, 2004
* Updated, April, September 2006
* FSI and CHEX added April 2009
* nondiagonal FSI is added May 2009
* H3 case is added         April 2013
********************************************************************** 
        implicit none
        real*8 ei,q2,q0,pfx,pfy,pfz,pr2,thr2,phir2,crs0,crs,crs_sf
        real*8 crspl, crsmn
        integer ktf,ktr2,ktr3,icon,ifl
        integer ksf,ksr2,ksr3
        integer k1s,k2s
        real thr2_s,phir2_s
	common/photon_d/      q_v_x,  q_v_y,  q_v_z,q_v
        real q_v_x,  q_v_y,  q_v_z,q_v
	common/knocked_d/     p_f_x,  p_f_y,  p_f_z,pf
        real  p_f_x,  p_f_y,  p_f_z,pf
	common/rescattered_d/ p_r2_x,  p_r2_y,  p_r2_z,p_r2
        real  p_r2_x,  p_r2_y,  p_r2_z,p_r2,e_r2
        real,  dimension(3) :: pv_f,pv_r2
*********************************************************************
        common/par/pi_s,pm_s,pmp,pmn,tm,eb
        real pi_s,pm_s,pmp,pmn,tm,eb
        common/proton_num/zpro
        real zpro
	common/electron/ei_s,er_s,ue_s
        real ei_s,er_s,ue_s
      	common/photon/q2_s,q0_s,qv_s
        real q2_s,q0_s,qv_s
        common/helium_spin/kjhe3
        integer kjhe3
	complex, dimension(0:3) :: jhe3_s
        real,    dimension(0:3) :: jhe3_re, jhe3_im
        real,    dimension(3)   :: sv_a,sv_f

        complex*16, dimension(0:3) :: jhe3_s_db
        complex*16, dimension(0:3,0:3) :: wa_s
        complex*16, dimension(0:3,0:3) :: wa_sum
       
***********************************************************************
        real e_f,thf,phif
        real p_m_x,p_m_y,p_m_z,p_m, T_res,thm,phim
        real p_r3_x,p_r3_y,p_r3_z,p_r3,E_r3,thr3,phir3
        real em,em0
        real*8 er,esc,ue,sn,qv
        real*8 cs_th_eq,sn_th_eq,th_eq,phi_eq
        real*8 cs_th_escq,sn_th_escq,phi_escq
        complex*16 jejhe3
        real*8 s_un,a_un,sum,absM2,fkin,part,sf,sf0,s_pol
        real*8 signp
        real*8 absM2new
        real*8 sfc,v_l,v_t,v_tt,v_tl,w_l,w_t,w_tt,w_tl,w_t_pr,w_tl_pr
        real*8 ga_2,ga_1
        real*8 tm_db
*********************************************************************
*   arguments of j_elm_pol in double precission
********************************************************************
        common/parm/pi,pm,dm,me  ! this goes to the calculation of j_N
	real*8 pi,pm,dm,me
        real*8, dimension(3) :: k1(3),k2(3),q(3),s1(3),s2(3)
        complex*16, dimension(0:3,2,2):: j_e
        complex*16, dimension(0:3,0:3) :: le
	integer nuc,ipl1,ipl2
        integer mu,nu
        real*8 s2m,s1m
        real*8 hei,sa,thsa,phisa
        real*8 ke_sa,ke_pr_sa,phi_1,phi_2
        real*8 v_tlpr,v_tpr,w_tlpr,w_tpr,a_pol,a_pl,a_mn,asym

        real GEn_s,GMn_s
        real *8 tau,tn,GE,GM,pol1,pol2,delt,sigm,asymt

*********************************************************************
        real cs_fr2,fc,fc1
*        real fc1
*        real fc
        common/alpha/alpha
        real*8 alpha
        real*8 zero
        zero = 0.0d+0

        ifl = 0
        crs = 0.0d+0
        crspl = zero
        crsmn = zero


*********************************************
* Initialization
**********************************************
 	if(icon.eq.-1)then
        pi = dacos(-1.0d+0)
        alpha = 1.0d+0/137.0d+0
        pm  = 0.938279d+0         
        dm  = 1.875628d+0
	me  = 0.000511d+0

        sv_f = 0.0
        sv_a = 0.0

       call j_he3(q0_s,qv_s,pv_f,ktf,ksf,sv_f,pv_r2,ktr2,ksr2,ktr3,ksr3,
     &                  kjhe3,sv_a,jhe3_s,icon)

*******************************************************************************
*  setting up  the sping projection of He3  1/2 effective in sa direction
*******************************************************************************
        icon = 0 
        kjhe3  = 1
       call j_he3(q0_s,qv_s,pv_f,ktf,ksf,sv_f,pv_r2,ktr2,ksr2,ktr3,ksr3,
     &                  kjhe3,sv_a,jhe3_s,icon)

        nuc = 0

	q2 = zero
	q  = zero
	k2 = zero
	ipl2 = 0
	s2 = zero
	k1 = zero
	ipl1 = 0
	s1 = zero
	j_e = dcmplx(zero,zero)

        call j_elm_pol_db(nuc,q2,q,k2,ipl2,s2,k1,ipl1,s1,j_e)
 
	return
	endif

*********************************************
* Calculation
**********************************************

**************************************
*       INPUT      
**************************************
*       electron & photon
**************************************
	ei_s  =  sngl(ei)
 	q2_s  =  sngl(q2)
	q0_s  =  sngl(q0)
 	er  = ei - q0 
 	sn = dsqrt(q2/4.0d+0/ei/er)
	if(sn**2.gt.1.0d+0)goto 111
	ue = 2.0d+0 * dasin(sn)
  	qv  = dsqrt(q2 + q0**2)
        
        er_s = sngl(er)
        ue_s = sngl(ue)
        qv_s = sngl(qv)
*************************************************
*     Descartes
*************************************************
*************************************************
* computing electron current
*************************************************
        esc = er
****************************
*  definning incident electron 3-momentum 
*  in the reference frame with z||q
**************************************************
	cs_th_eq = (ei - esc*dcos(ue))/qv
	sn_th_eq =       esc*dsin(ue) /qv
	phi_eq = 0.0d+0
	k1(1) = 0.0d+0; k1(2) = 0.0d+0; k1(3) = ei

	k1(1) = ei*sn_th_eq*dcos(phi_eq)
	k1(2) = ei*sn_th_eq*dsin(phi_eq)
	k1(3) = ei*cs_th_eq

	th_eq = dacos(cs_th_eq)
**************************************************
*  definning scattered electron 3-momentum 
*  in the reference frame with z||q
**************************************************
	cs_th_escq = (ei*dcos(ue)-esc)/qv
	sn_th_escq =  ei*dsin(ue)/qv
	phi_escq   = 0.0d+0
	if((th_eq + ue).ge.pi)then
	phi_escq   = pi
	endif

	if(phi_escq.gt.0.0)then
	print *,"attention"
	stop
	endif
	k2(1) = esc*dsin(ue); k2(2) = 0.0; k2(3) = esc*dcos(ue) 

	k2(1) = esc*sn_th_escq*dcos(phi_escq)
	k2(2) = esc*sn_th_escq*dsin(phi_escq)
	k2(3) = esc*cs_th_escq

*************************************************
*  transferred momentum
************************************************* 
        q(1:3)  = k1-k2

 
**************************************
*  electromagnetic current
******************************************************************************
*        s2m = s2(1)**2 + s2(2)**2 + s2(3)**2
*        ipl2 = 1
*        if(s2m.lt.0.1)ipl2 = 0

        ipl2 = 0 ! no explicit polarization of final electron is taken into account

*        s1m = s1(1)**2 + s1(2)**2 + s1(3)**2
*        ipl1 = 1
*        if(s1m.lt.0.1)ipl1 = 0
        
        ipl1 = 0 ! no explicit polarization of initial electron is taken into account

        s2 = zero
        s1 = zero
        nuc = 3

*****************************************************************************
* Calculating electromagnetic current of electron
*****************************************************************************
        call j_elm_pol_db(nuc,q2,q,k2,ipl2,s2,k1,ipl1,s1,j_e)
*****************************************************************************


*****************************************************************************
* Calulating kinematics of hadronic part
*****************************************************************************

*************************************
*       virtual photon
*************************************
 	q_v   = qv_s	
        q_v_x = 0.0
	q_v_y = 0.0
	q_v_z = q_v

************************************** 
*       knocked out nucleon momentum
*       with respect to vrt. photon
**************************************
 	p_f_x = sngl(pfx) 
  	p_f_y = sngl(pfy) 

************************************** 
*       recoil nucleon 
*       with respect to vrt. photon
**************************************
	p_r2    =  sngl(pr2)
 	p_r2_x  =  sngl(p_r2 * dsin(thr2)*dcos(phir2))
        p_r2_y  =  sngl(p_r2 * dsin(thr2)*dsin(phir2))
        p_r2_z  =  sngl(p_r2 * dcos(thr2))
	e_r2    =  sqrt(pm_s**2 + p_r2**2)

********************************************
*       Determination of z component of of 
*       three body break_up
********************************************
 	call kine_pfz(ifl )
	if(ifl.eq.1)goto 111
        pfz   = p_f_z        
 	pf   = sqrt(p_f_x**2 + p_f_y**2 + p_f_z**2)
	e_f    = sqrt(pm_s**2 + pf**2)
      	call polar(p_f_x,p_f_y,p_f_z,pf,thf,phif)

************************************** 
*       missing nucleon momentum
*       with respect to vrt. photon
**************************************
        p_m_x =  p_f_x - q_v_x	
        p_m_y =  p_f_y - q_v_y	
        p_m_z =  p_f_z - q_v_z	
        p_m   = sqrt(p_m_x**2 + p_m_y**2 + p_m_z**2) 
        T_res = sqrt((2.0*pm_s)**2 + p_m**2) - 2.0*pm_s

      	call polar(p_m_x,p_m_y,p_m_z,p_m,thm,phim)

********************************************
* Calculating r3 momentum and energy
********************************************
        p_r3_x = -(p_m_x + p_r2_x)
        p_r3_y = -(p_m_y + p_r2_y)
        p_r3_z = -(p_m_z + p_r2_z)

        p_r3 = sqrt(p_r3_x**2 + p_r3_y**2 + p_r3_z**2)
        E_r3 = sqrt(pm_s**2 + p_r3**2)
      	call polar(p_r3_x,p_r3_y,p_r3_z,p_r3,thr3,phir3)


*************************************
*  Missing energy 
*************************************
        Em  = E_r2 + E_r3 - 2.0*pm_s + eb  - T_res
        em0 = em + T_res                

*******************************************************************
*       Calculation of cross section
*******************************************************************

*******************************************************************
*       Calculating Electromagnetic Current of He3
********************************************************************
        pv_f(1)  = p_f_x
        pv_f(2)  = p_f_y
        pv_f(3)  = p_f_z
        
        

        pv_r2(1) = p_r2_x
        pv_r2(2) = p_r2_y
        pv_r2(3) = p_r2_z



*        icon = 1
*        print *,"zpro",zpro,ktr3
        ktr3 =  1 - (ktf+ktr2)
        if(zpro.gt.1.5)ktr3 =  1 - (ktf+ktr2)
        if(zpro.lt.1.5)ktr3 =  -1 - (ktf+ktr2)

        sum = 0.0

         wa_sum   = (0.0,0.0)

*******************************************************************
*  Three dimensional spin vector of the nucleus
******************************************************************
         sv_a(1)  = sin(sngl(thsa))*cos(sngl(phisa))
         sv_a(2)  = sin(sngl(thsa))*sin(sngl(phisa))
         sv_a(3)  = cos(sngl(thsa))



*******************************************************************
*  Three dimensional spin vector of kncoked-out nucleon
******************************************************************
         sv_f(1) = 0.0
         sv_f(2) = 0.0
         sv_f(3) = 1.0

	 do ksf  = -1,1,2  ! spin of the fast nucleon !knocked-out nucleon            
	 do ksr2 = -1,1,2  ! spin of the slow nucleon !recoil nucleon
         do ksr3 = -1,1,2  ! spin of the second slow nucleon ! recoil nucleon 
        

********************************************************************************
*   calculation of the electromagnetic current of He3
********************************************************************************
         jhe3_s = 0.0
       call j_he3(q0_s,qv_s,pv_f,ktf,ksf,sv_f,pv_r2,ktr2,ksr2,ktr3,ksr3,
     &                  kjhe3,sv_a,jhe3_s,icon)

         jhe3_re = 0.0; jhe3_im = 0.0

	 jhe3_re =  real(jhe3_s)
	 jhe3_im = aimag(jhe3_s)

***********************************************************************
* Making electromagnetic current of He3  complex double precission
***********************************************************************
	 jhe3_s_db = dcmplx(dble(jhe3_re),dble(jhe3_im))


***********************************************************************
*  Calculating electromagnetic tensor of he3 nucleus  W^\mu^\nu
***********************************************************************

         do mu = 0,3
         do nu = 0,3
         wa_s(mu,nu) = conjg(jhe3_s_db(mu))*jhe3_s_db(nu)
         enddo
         enddo


*******************************************************************************
*  setting up  the spin projection of He3 -1/2
*******************************************************************************
*         jhe3_s_d = 0.0
*         icon = 0 
*         kjhe3  = -1
*         call j_he3(q0_s,qv_s,pv_f,ktf,ksf,pv_r2,ktr2,ksr2,ktr3,ksr3,
*     &                  kjhe3,jhe3_s_d,icon)


*         icon = 1
*         call j_he3(q0_s,qv_s,pv_f,ktf,ksf,pv_r2,ktr2,ksr2,ktr3,ksr3,
*     &                  kjhe3,jhe3_s_d,icon)


*         jhe3_re = 0.0; jhe3_im = 0.0

*	 jhe3_re =  real(jhe3_s_d)
*	 jhe3_im = aimag(jhe3_s_d)

*	 jhe3_d = dcmplx(dble(jhe3_re),dble(jhe3_im))

*         do mu = 0,3
*         do nu = 0,3
*         wa_d(mu,nu) = conjg(jhe3_d(mu))*jhe3_d(nu)
*         enddo
*         enddo

         
*         saz = dcos(tsa)
         
*         do mu = 0,3
*         do nu = 0,3
*          wa(mu,nu) = wa_u(mu,nu) + wa_d(mu,nu)
*         enddo
*         enddo


         wa_sum   = wa_sum + wa_s
*         wa_u_sum = wa_u_sum + wa_u
*         wa_d_sum = wa_d_sum + wa_d


***********************************************************************
*     Calculating the product j_e j_he3
***********************************************************************

***************************************************************************
        


*	 jejhe3 = j_e(0,k2s,k1s)*jhe3_u(0) - j_e(1,k2s,k1s)*jhe3_u(1) 
*     &          - j_e(2,k2s,k1s)*jhe3_u(2) - j_e(3,k2s,k1s)*jhe3_u(3)
	 

*	 absM2 = abs(jejhe3)**2   !a
****************************************************************************
*         print *,absM2n,absM2

	 enddo 
	 enddo 
         enddo


         sum = 0.0

	 do k2s = 1,2     ! spin of the scattered electron
	 do k1s = 1,2     ! spin of the initial electron


***************************************************************
* Electromagnetic tensor of electron
***************************************************************
         do mu = 0,3
         do nu = 0,3
         le(mu,nu) = conjg(j_e(mu,k2s,k1s))*j_e(nu,k2s,k1s)
         enddo
         enddo


************************************************************
*  Calculation of |M^2|
************************************************************

         absM2new = 0.0
         do mu = 0,3
         do nu = 0,3
         absM2new =absM2new +wa_sum(mu,nu)*le(mu,nu)*signp(mu)*signp(nu)
         enddo
         enddo
         absM2 = absM2new
	 sum   = sum + absM2

         enddo   ! end of the sum by initial electron polarization
         enddo   ! end of the sum by final   electron polarization
*         print *,'sum',sum
*         s_un = (1.0d+0/2.0d+0)*(1.0d+0/2.0d+0)*sum

************************************************************
* Averaging over the polarizations of initial electron
*
* we dont need to average over the polarization of he3
* since we choosed only one polarization of the He3
* this is effectively correspond to the unpolarized case
* since we sum over all the polarizations of all the particles.
************************************************************
         s_un = (1.0d+0/2.0d+0)*sum
*****************************************************
* |M|^2 = e^4/Q^4 * s_un
*****************************************************
         
*********************************************************************
*   One divides only by the number of spin polarization of electron
*   sine He3's polarization is fixed
*********************************************************************
*         print *,'s_un',s_un
         
********************************************************************
* Alternative way of calculating M2 is through lepton and nuclear 
* structure functions
*********************************************************************

         sfc  = 4.0d+0*esc*ei*dcos(ue/2.0d+0)**2
         v_l  = q2**2/qv**4
         v_t  = q2/2.0d+0/qv**2 + dtan(ue/2.0d+0)**2
         v_tt = q2/2.0d+0/qv**2
         v_tl = q2/qv**2*dsqrt(q2/qv**2 + dtan(ue/2.0d+0)**2)


         w_l  =         Real(wa_sum(0,0))
         w_t  =         Real(wa_sum(2,2) + wa_sum(1,1))
         w_tt =         Real(wa_sum(1,1) - wa_sum(2,2))
         w_tl = -2.0d+0*Real(wa_sum(0,1))



        a_un =   sfc*
     &          (v_l*w_l + v_t*w_t + v_tl*w_tl + v_tt*w_tt)


         v_tlpr = dtan(ue/2.0d+0) * q2/qv**2
         v_tpr  = dtan(ue/2.0d+0) * dsqrt(q2/qv**2 + dtan(ue/2.0d+0)**2)

         w_tlpr = -2.0d+0*aimag(wa_sum(2,0))
         w_tpr  = -2.0d+0*aimag(wa_sum(1,2))

         a_pol = sfc*(v_tlpr*w_tlpr + v_tpr*w_tpr)

         a_pl = a_un + sa*a_pol
         a_mn = a_un - sa*a_pol

*         asym = a_pol/a_un
************************************************************
*   Asymmetry for the case of isolated nucleon at rest
************************************************************
*      print *,"thsa2",thsa,q2_s

*         tau = q2/(4.0*pm**2)
*         tn  = dtan(ue/2.0d+0)
*         call neutron_formfactors(q2_s,GEn_s,GMn_s)


*         GE = dble(GEn_s)
*         GM = dble(GMn_s)
         
*         pol1 = dsqrt(tau*(1.0+(1.0+tau)*tn**2))*GM**2*dcos(thsa)
*         pol2 = GM*GE*dsin(thsa)*dcos(phisa)
*         delt = -2.0*tn*dsqrt(tau/(1.0+tau))*(pol1+pol2)

*         sigm = (GE**2+tau*GM**2)/(1.0+tau) + 2.0*tau*tn**2*GM**2
         
*         print *,"delt",delt,"sigm=",sigm
*         asymt = delt/sigm
*         print *,"asymt",asymt,gm,ge
****************************************************************





*         tm_db = dble(tm)

*       ga_2=-(1.0d+0/2.0d+0)*(tm_db**2/qv**3)*(q0*w_tl_pr -  qv* w_t_pr)
*       ga_1=-(1.0d+0/2.0d+0)*(tm_db   /qv**3)*(q0*qv*w_t_pr +q2*w_tl_pr)

*        ke_sa = ei*(cs_th_eq*dcos(thsa)+sn_th_eq*dsin(thsa)*dcos(phisa))
*        ke_pr_sa =esc*(cs_th_escq*dcos(thsa) + 
*     &                 sn_th_escq*dsin(thsa)*dcos(phi_escq)*dcos(phisa))

*        phi_1 = ke_sa + ke_pr_sa
*        phi_2 = esc*ke_sa - ei*ke_pr_sa

*        s_pol = sfc* 2.0d+0*dtan(ue/2.0d+0)**2*hei*sa*(ga_1/tm_db*phi_1  
*     &                                     - 2.0d+0*ga_2/tm_db**2*phi_2)

*         print *,"wa_sum",wa_u_sum(2,2),wa_d_sum(2,2)
*         print *,"wa_pr",w_t_pr,w_tl_pr,ga_1,ga_2
*         print *,"phi",phi_1,phi_2
*         print *,"s",s_un,s_pol
*********************************************************************
*       eN cross section in nanobarn 
*********************************************************************
*       ireg = 22
*       if(ktf.eq.1.and.ktr2.eq.-1)ireg=21
*       if(ktf.eq.-1.and.ktr2.eq.1)ireg=12
*       sigma   = sigma_en(p_m_x,p_m_y,p_m_z,ireg)


************************************************
*       Kinematic factor
*************************************************	
         thr2_s   = sngl(thr2)
         phir2_s  = sngl(phir2)
         cs_fr2 = cos(thf)*cos(thr2_s)+sin(thf)*sin(thr2_s)
     &            *cos(phif-phir2_s)
*        if(iout.eq.1)then
	fc1 = (p_r2 - qv*cos(thr2_s) + pf*cs_fr2)/e_r3
        fc  = abs(p_r2/e_r2 + fc1)
*        elseif(iout.eq.2)then
*	fc1 = (p_f - qv*cos(thf) + p_r2*cs_fr2)/e_r3
* 	fc = abs(p_f/e_f + fc1)
*	endif
                     fkin  = 0.0d0
        if(fc.gt.1.0e-7)fkin  = dble(pf**2 * p_r2**2 / fc)
****************************************
	sf0 = 9.0d+0 * alpha**2/q2**2 * esc/ei *dble(1.0/(4.0*tm*e_f))
*	sf0 = alpha**2/q2**2 * esc/ei *dble(1.0/(4.0*pm*e_f))
	sf  = sf0*0.389385*1000.*1000.
*        print *,'fkin',fkin

**********************************************************************
*  Cross section is d\sigma/ dEe d\omega_e dp_f d\omega_f d\omega_r2
        crs        = s_un * fkin * sf
        crs_sf     = a_un * fkin * sf
        crspl      = a_pl * fkin * sf
        crsmn      = a_mn * fkin * sf
**********************************************************************
*        print *,"crs",crs,(crspl+crsmn)/2.0
*        print *,"asymt_mihatel",asymt
 111    continue
        return
        end

*       ____________________        
        function signp(i)
        real*8 signp
        integer i
        signp = 1.0d+0
        if(i.ne.0)signp=-1.0d+0
        return
        end
*       ________________________
	subroutine kine_pfz(ifl)
**********************************************************
*       subroutine calculate the z_component of pf       *
**********************************************************
        common/par/pi,pm,pmp,pmn,am,eb    
       	common/photon/q2,q0,qV
	common/photon_d/      q_v_x,  q_v_y,  q_v_z,q_v
	common/knocked_d/     p_f_x,  p_f_y,  p_f_z,p_f
	common/rescattered_d/ p_r_x,  p_r_y,  p_r_z,p_r
	ifl   = 1
 	e_r   = sqrt(pm**2 + p_r**2) 
	q_pr  = q_v_x*p_r_x + q_v_y*p_r_y + q_v_z*p_r_z 
   	pfr_t = p_f_x*p_r_x + p_f_y*p_r_y
	pft2  = p_f_x**2    + p_f_y**2
	tm    = -q2 - 2.0*q0*e_r + 2.0*q_pr + pm**2

	arm   = tm - 2.0*pfr_t + pm**2 + 2.0*am*(q0-e_r) 
	arm_a = arm + am**2 - pm**2
	arm_b = 2.0*(qv - p_r_z)
	arm_g = 2.0*(q0 + am - e_r)

	A     = arm_b**2 - arm_g**2
	B     = 2.0*arm_a*arm_b
	C     = arm_a**2 - (pm**2+pft2)*arm_g**2
	if(A.eq.0.0)then
	p_f_z = -C/B
 	else
   	call  squar_eq(a,b,c,x1,x2,i_test)
	if(i_test.eq.1)return
*	type *,'xx',x1,x2
     	p_f_z1 = x1
	test1  = (arm_a + p_f_z1*arm_b)/arm_g  
   	p_f_z2 = x2
	test2  = (arm_a + p_f_z2*arm_b)/arm_g  

	p_f_z  = p_f_z1		
	if(test1.lt.pm.or.p_f_z1.lt.0.0)then
 	if(test2.lt.pm)return
	p_f_z  = p_f_z2
	elseif(test2.gt.pm.and.p_f_z2.gt.0.0)then
	if(p_f_z2.gt.p_f_z1)p_f_z = p_f_z2
	endif
	endif
 	ifl   = 0	
 	return
 	end



*     ___________________________________________
      subroutine kin_emth(q2,q0,em,p_m,thm,itest)
      common/par/pi,pm,pmp,pmn,tm,eb
      itest = 1
      if(p_m.eq.0.0)then
      thm   = 0.0
      itest = 0
      return
      endif
      qv = sqrt(q2 + q0**2)
      em0 = em + p_m**2/4.0/pm
      a_arm = q0 - em0
      hamarich = a_arm**2 + 2.0*pm*a_arm - p_m**2 - qv**2
      haytarar = 2.0*p_m*qv
      cs_thm = hamarich/haytarar
      if(cs_thm**2.gt.1.0)return
      thm   = acos(cs_thm)
      itest = 0
      return
      end

*     ____________________________________________
      subroutine kin_emq0(q2,p_m,thm,em,q0,itest)
      common/par/pi,pm,pmp,pmn,tm,eb
      itest = 1
      em0 = em + p_m**2/4.0/pm
      tlpm  = pm - em0
      if(tlpm.le.0.0)return

      pmz = p_m*cos(thm)
      if(pmz**2.le.0.000001)then
      q0 = (pm**2 - tlpm**2 + p_m**2 + q2)/(2.0*tlpm)
      if(q0.le.0.0)return
      itest = 0
      return
      endif
*     _____________________________________
      arm_a = tlpm**2 - pm**2 - p_m**2 - q2
      A = 4.0*(tlpm**2 - pmz**2)
      B = 4.0*tlpm*arm_a
      C = arm_a**2 - 4.0*pmz**2*q2

      call squar_eq(A,B,C,x1,x2,i_test)
      if(i_test.eq.1)return
      test1 = (2.0*tlpm*x1 + arm_a)/pmz
      test2 = (2.0*tlpm*x2 + arm_a)/pmz
      q0 = x1
      if(x1.le.0.0.or.test1.le.0.0)then
      q0 = x2
      if(x2.le.0.0.or.test2.le.0.0)return
      itest = 0
      return
      endif
      if(x2.gt.0.0.and.test2.gt.0.0)then 
      write(6,*)"two solutions!"
      endif
      itest = 0
      return
      end
*       _______________________________________
        SUBROUTINE SQUAR_EQ(A,B,C,X1,X2,I_TEST)
        I_TEST = 0
        DSKM  = B**2 - 4.0*A*C
        IF(DSKM.LT.0.0)THEN
                IF(DSKM.GT.-1.0E-4)THEN
                DSKM = 0.0
                GOTO 111
                ELSE
                I_TEST = 1
                ENDIF
        RETURN
        ENDIF
111     X1 = ( -B + SQRT(DSKM) ) / 2.0 / A
        X2 = ( -B - SQRT(DSKM) ) / 2.0 / A
        RETURN
        END


*       subroutine j_he3(q0,qv,jhe3,ktm,p_m,thm,phim,
*     &                   ktr,pr,thr,phir,spec,icrs,icon)


       subroutine j_he3(q0,qv,p_f,ktf,ksf,sv_f,p_r,ktr,ksr,kts,kss,
     &                  kjhe3,sv_a,jhe3,icon)
  
**********************************************************************
*
* 
* q0      - virtual photon energy
* qv      - virtual photon momentum
* p_f(3)  - three vector of knocked-out nucleon 
* ktf     - isospin of the knocked-out nucleon 1-proton, -1 neutron
* ksf     - spin of the knocked-out nucleon 1-up, -1 down
* sv_f(3) - three dimensional spin vector of  knocked-out nucleon
* p_r(3)  - three vector of recoil nucleon
* ktr     - isospin of recoil nucleon 1-proton -1 neutron 
* ksr     - spin of recoil nucleon 1-up -1 down 
* kts     - isospin of undetected recoil nucleon 1-proton -1 neutron 
* kss     - spin of undetected recoil nucleon 1-proton -1 neutron 
* kjhe3   - total spin projectsion of 3He target 1->1/2, -1 -1/2     - input
* sv_a(3) - three dimensional spin vector of he3
* jhe3(0:3) - he3 electromagnetic current 0,1,2,3 - components
*
*
* ktm  - isospin projection of   m  nucleon 1->1/2,-1 -1/2        - input
* p_m  - momentum of             m   nucleon  GeV/c               - input
* thm  - polar angle of          m  nucleon                       - input
* phim - azimuthal angle         m  nucleon                       - input
* ktr  - isospin projection of   r  nucleon 1->1/2,-1 -1/2        - input  
* pr   - momentum of             r  nucleon  GeV/c                - input
* thr  - polar angle             r  nucleon                       - input
* phir - azimuthal angle         r  nucleon                       - input
* ireg - ij type of the  m and r    nucleons
*        22-proton-proton, 21-proton-neutron, 12-neutron-proton
*
* icon - -1 - initialize, 0 - reset then spin projection of He3 
*         1- only pwia, 2-only A0s, 12 only A0+A0s
*
* The azimuthal angles are defined in the reference frame where 
* q || z and xz is the (eq), e is the initial electron
*
*
* Misak Sargsian
* FIU, September, 2003
*
*
********************************************************************** 
        common/par/pi,pm,pmp,pmn,tm,eb
	common/proton_num/z	
	common/atomic_num/an
	common/missing/ p_m_x,  p_m_y,  p_m_z
        common/recoil_r/p_r_x,  p_r_y,  p_r_z
	common/recoil_s/p_s_x,  p_s_y,  p_s_z
        dimension pvec_m(3), pvec_r(3),pv1(3),pv2(3)
        dimension p_f(3),p_r(3),p_m(3),q(3)
        real,    dimension(3)   :: sv_a,sv_f
	complex, dimension(0:3) :: jhe3
        complex, dimension(0:3,2,2):: jNm,jNmchex

        common/sfun/ wa_re,  wa_im
	integer sf,s1,sfc,sfpr
        complex A0,A0s,A1,A1s,A1chex,A2
        common/parms/pi_s,pm_s,dm_s
        
         jhe3 = cmplx(0.0,0.0)
 	if(icon.eq.-1)then
****************************************************************
*       initialization 
****************************************************************
        pi  = acos(-1.0)       
        pm  = 0.938279         
        dm  = 1.875628
*        pmp = pm
*        pmn = 0.939565  
*	 an  = 3.0
*	 z   = 2.0
*	 eb  = 0.007718109 !0.0085
*	 am  = z*pm + pmn  - eb
*        tm  = am 
	pi_s = pi
	pm_s = pm
	dm_s = dm

*        xxx = delta_fun(ispin,pl,3)  ! reading phase shift data       
        call A0s_wf_full(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,kj,
     &                   a0s_re,a0s_im,3)  !reads Bystricky amplitudes
                                     
        iflag = 1                    ! reading the wave function data
        call wf_he3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,kj,
     x                                        wfre,wfim,iflag)
 

	nuc = 0
	q2 = 0.0
	p_f = 0.0
	p_r = 0.0
	q = 0.0
*        call  j_N_matrix(nuc,q2,q,p_f,p_m,jNm)
        call  j_N_matrix_s(nuc,q2,q,p_f,sv_f,p_m,sv_a,jNm)

       return
****************************************************************
*       initialization of the spin projection of 3He
****************************************************************
        elseif(icon.eq.0)then

        iflag = 2   ! reseting the projection of total spin 1/2
        kj =  kjhe3
        call wf_he3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,kj,
     x                                        wfre,wfim,iflag)
	return
	endif
****************************************************************
*       end initialization 
****************************************************************


****************************************************************
*       INPUT      
****************************************************************

************************************** 
*       spectator/recoil nucleon
*       anglesare  with respect to q
**************************************
 	p_r_x  =  p_r(1) 
        p_r_y  =  p_r(2) 
        p_r_z  =  p_r(3)
        
        pvec_r(1) =  p_r_x
        pvec_r(2) =  p_r_y
        pvec_r(3) =  p_r_z
************************************** 
*       Missing momenta
**************************************
	p_m_x  = p_f(1)
	p_m_y  = p_f(2)
	p_m_z  = p_f(3) - qv
	p_m    = sqrt(p_m_x**2 + p_m_y**2 + p_m_z**2)
        pvec_m(1) =  p_m_x
        pvec_m(2) =  p_m_y
        pvec_m(3) =  p_m_z
      
************************************** 
*       Spectator momenta
**************************************
        p_s_x =  -(p_m_x + p_r_x)
        p_s_y =  -(p_m_y + p_r_y)
        p_s_z =  -(p_m_z + p_r_z)
        p_s   = sqrt(p_s_x**2 + p_s_y**2 + p_s_z**2)
        e_s   = sqrt(pm**2 + p_s**2)
*******************************************************************
*      Virtual photon  momenta
*******************************************************************
	q2   = qv**2 - q0**2
	q(1) = 0.0
	q(2) = 0.0
	q(3) = qv
********************************************************************

      if(icon.eq.5.or.icon.eq.15.or.icon.eq.135.or.icon.eq.1356)then  ! chex - contribution for elm
       call  j_N_matrix_s(-ktf,q2,q,p_f,sv_f,pvec_m,sv_a,jNmchex)
*       jNmchex = jNm
       endif

*       call  j_N_matrix(ktf,q2,q,p_f,pvec_m,jNm)
       call  j_N_matrix_s(ktf,q2,q,p_f,sv_f,pvec_m,sv_a,jNm)



 
***********************************************
*  Spin of the knocked-out nucleon
***********************************************
	             sf  = 1
                     sfc = 2
	if(ksf.eq.-1)then
                     sf  = 2
                     sfc = 1
                     endif

	do mu=0,3      ! running \mu components 

        A0      = cmplx(0.0,0.0)
        A0s     = cmplx(0.0,0.0)
        A1      = cmplx(0.0,0.0)
        A1s     = cmplx(0.0,0.0)
        A1chex  = cmplx(0.0,0.0)
        A1nd    = cmplx(0.0,0.0)
        A2      = cmplx(0.0,0.0)        


         wf_re   = 0.0
         wf_im   = 0.0
           
     
      do ksm = -1,1,2  
	              s1 = 1
	 if(ksm.eq.-1)s1 = 2
      
*        do ksr = -1,1,2
*         do kss = -1,1,2
 
         wf0_re  = 0.0
         wf0_im  = 0.0
         wfs_re  = 0.0
         wfs_im  = 0.0
         wf1_re  = 0.0
         wf1_im  = 0.0
         wf1s_re = 0.0
         wf1s_im = 0.0
         wf1ch_re = 0.0
         wf1ch_im = 0.0
         wf1nd_re = 0.0
         wf1nd_im = 0.0
         wf2_re  = 0.0
         wf2_im  = 0.0

         
      if(icon.eq.1.or.icon.ge.12)then   ! A0  - contribution
      ktm = ktf

      if(z.gt.1.5)then  ! he3
      call wf_he3_nb(ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,kjhe3,
     x                                        wf0_re,wf0_im,0)       
      elseif(z.lt.1.5)then !h3
      call wf_h3_nb(ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,kjhe3,
     x                                        wf0_re,wf0_im,0)       
      endif


*      wf0_re = 1.0
*      wf0_im = 0.0
*here
      A0 = A0 +  jNm(mu,sf,s1)*cmplx(wf0_re,wf0_im)
*      A0 = A0 +   cmplx(wf0_re,wf0_im)

      endif
      

      if(icon.eq.2.or.icon.eq.12.or.icon.eq.123.or.icon.ge.123456)then    ! A0s - contribution
         ktm = ktf
      call A0s_wf_full(ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,kjhe3,
     &                                      wfs_re,wfs_im,0)
      A0s = A0s +  jNm(mu,sf,s1)*cmplx(wfs_re,wfs_im)

      endif


      if(icon.eq.3.or.icon.eq.13.or.icon.eq.123.or.icon.ge.134)then    ! A1 - contribution
      ifl = 1 !1- pole case, 2-pv, 0-both
      md  = 2 !method of integration 1,2,3,4, 2- Gadap
      ktm = ktf
      call A1_wf_sum(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,kjhe3,
     &                                       wf1_re,wf1_im,ifl,md)
      A1 = A1 +  jNm(mu,sf,s1)*cmplx(wf1_re,wf1_im)
      
      endif

      if(icon.eq.4.or.icon.eq.134.or.icon.eq.1234.or.icon.ge.123456)then    ! A1s - contribution
      ifl = 1 !1- pole case, 2-pv, 0-both
      md  = 2 !method of integration 1,2,3,4, 2- Gadap
      ktm = ktf
      call A1s_wf_sum(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,kjhe3,
     &                                         wf1s_re,wf1s_im,0)
      A1s = A1s +  jNm(mu,sf,s1)*cmplx(wf1s_re,wf1s_im)

      endif

      if(icon.eq.5.or.icon.eq.15.or.icon.eq.135.or.icon.ge.1356)then    ! chex - contribution
      ifl = 1 !1- pole case, 2-pv, 0-both
      md  = 2 !method of integration 1,2,3,4, 2- Gadap

      ivr = 10 !1021 !10021 !1021 !21 !10
      ktmc = -ktf
*      idn = 1
*      call A1_wf_chex_sum_ab(q0,qv,ksf,ktmc,ksm,pvec_m,ktr,ksr,pvec_r,
*     &             kts,kss,kjhe3,wf1ch_re_dn,wf1ch_im_dn,ifl,md,ivr,idn)
      idn = -1
      call A1_wf_chex_sum_ab(q0,qv,ksf,ktmc,ksm,pvec_m,ktr,ksr,pvec_r,
     &             kts,kss,kjhe3,wf1ch_re_nd,wf1ch_im_nd,ifl,md,ivr,idn)

*      ktm = ktf
*      call A1_wf_sum(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,kjhe3,
*     &                                       wf1ch_re,wf1ch_im,ifl,md)

      A1chex = A1chex+jNmchex(mu,sfc,s1)*cmplx(wf1ch_re_nd,wf1ch_im_nd)
*     &               +jNmchex(mu,sf,s1) *cmplx(wf1ch_re_dn,wf1ch_im_dn)

*      print *,"aha",wf1ch_re,wf1ch_im


*      A1chex = A1chex +  jNm(mu,sf,s1)*cmplx(wf1ch_re,wf1ch_im)
*      A1chex = A1chex +  jNm(mu,sf,s1)*cmplx(1.,1.)
*      write(14,*)sf,s1,jNmchex(mu,sf,s1),wf1ch_re,wf1ch_im
*      A1chex = A1chex +  cmplx(wf1ch_re,wf1ch_im)
      endif

      if(icon.eq.6.or.icon.eq.16.or.icon.eq.136.or.icon.ge.1356)then    ! nd - contribution
      ifl = 1 !1- pole case, 2-pv, 0-both
      md  = 2 !method of integration 1,2,3,4, 2- Gadap

      ktm = ktf
      call A1nd_wf_sum(q0,qv,ksf,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,
     &                                kjhe3,wf1nd_re,wf1nd_im,ifl,md)

*      print *,"wfs",wf1nd_re,wf1nd_im

                 sfpr=1
      if(sf.eq.1)sfpr=2
      A1nd = A1nd +  jNm(mu,sfpr,s1)*cmplx(wf1nd_re,wf1nd_im)
     
*      print *,"A1nd",wf1nd_re,wf1nd_im
      endif


      if(icon.eq.137.or.icon.eq.1357.or.icon.eq.13567.or.
     & icon.eq.1234567)then    ! A2 - contribution
      mtd = 1
      ktm = ktf
      if(ktm.eq.1.and.ktr.eq. 1)irs = 221
      if(ktm.eq.1.and.ktr.eq.-1)irs = 212
      if(ktm.eq.-1)             irs = 122
*      call A2_wf(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe3,
*     &                                 wf2_re,wf2_im,mtd,irs)
      call A2_wf(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,kjhe3,
     &                                 wf2_re,wf2_im,mtd,irs)
      A2 = A2 +  jNm(mu,sf,s1)*cmplx(wf2_re,wf2_im)
*      write(6,*)wf2_re,wf2_im
*      ifl = 1 !1- pole case, 2-pv, 0-both
*      md  = 2 !method of integration 1,2,3,4, 2- Gadap
*      call A1s_wf_sum(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe3,
*      &                                         wf1s_re,wf1s_im,0)
      endif

      
*      endif


*      wf_re = wf0_re + wfs_re + wf1_re  + wf1s_re + wf2_re 
*      wf_im = wf0_im + wfs_im + wf1_im  + wf1s_im + wf2_im
*      spec = spec +  wf_re**2 + wf_im**2


      enddo
*      print *,A1chex,A1nd
      jhe3(mu) = A0 + A0s + A1 + A1s + A1chex + A1nd + A2
*      jhe3(mu) = A0 +  A1 +  A1chex + A1nd
*      print *,jhe3(mu)
      enddo
*     print *,"jhe3",jhe3
      return
      end


      subroutine A1_wf_sum(q0,qv,ktm,ksm,pvec_m,
     & ktr,ksr,pvec_r,kts,kss,jhe,wf1_re,wf1_im,ifl,md)
***************************************************************************
* Subroutine calculates the distorted due to single FSI wave function
* summed by the  number of spectator scatterrers.
*
* a1_re and a1_im - real and imaginary parts                  - output
*
* q0     - virtual photon energy                                 - input
* qv     - virtual photon momentum                               - input
* ktf    - isospin projection of knocked-out nucleon             - input 
* ksf    - spin projection of knocked-out nucleon                - input 
* pvec_f - three vector of knocked-out nucleon                   - input 
* ktm    - isospin projection of initial nucleon                 - input 
* ksm    - spin projection of initial  nucleon                   - input 
* pvec_m - three vector of initial nucleon                       - input 
*
* ktr    - isospin projection of rescattered nucleon             - input 
* ksr    - spin projection of rescattered  nucleon               - input 
* pvec_r - three vector of rescattered nucleon                   - input 
* kss    - spin projection of spectator  nucleon                 - input 
* jhe    - spin projection of 3He                                - input 
* ifl = 0 !1- pole case, 2-pvi, 0-both                           - input
* md  = 2 !method of integration for PVI 1,2,3,4, 2- Gadap       - input
*
*
*
****************************************************************************
      dimension pvec_m(3),pvec_r(3)
      common/proton_num/zpro	

      wf1_re = 0.0
      wf1_im = 0.0

      wf1_re_r = 0.0
      wf1_im_r = 0.0 
      wf1_re_s = 0.0
      wf1_im_s = 0.0 

      if(ktm.eq.1)then !knocked out is proton
                     irs = 22
      if(zpro.lt.1.5)irs = 121
      call  A1_wf(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,
     &                             wf1_re_r,wf1_im_r,md,ifl,irs)
                     irs = 21
      if(zpro.lt.1.5)irs = 122
      call  A1_wf(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,
     &                             wf1_re_s,wf1_im_s,md,ifl,irs)
**      print *,"ahairs"
                                            
      endif

      if(ktm.eq.-1)then !knocked out is neutron
*      write(6,*)q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,ifl
                     irs = 121
      if(zpro.lt.1.5)irs = 22
      call  A1_wf(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,
     &                             wf1_re_r,wf1_im_r,md,ifl,irs)
                    irs = 122
      if(zpro.lt.1.5)irs = 21
      call  A1_wf(q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,
     &                             wf1_re_s,wf1_im_s,md,ifl,irs)
*      write(6,*)"ahairrrr"
      endif
      wf1_re = wf1_re_r + wf1_re_s
      wf1_im = wf1_im_r + wf1_im_s
      return
      end


      subroutine A1nd_wf_sum(q0,qv,ksf,ktm,ksm,pvec_m,
     & ktr,ksr,pvec_r,kts,kss,jhe,wf1_re,wf1_im,ifl,md)
***************************************************************************
* Subroutine calculates the distorted due to single FSI wave function
* summed by the  number of spectator scatterrers.
*
* a1_re and a1_im - real and imaginary parts                  - output
*
* q0     - virtual photon energy                                 - input
* qv     - virtual photon momentum                               - input
* ktf    - isospin projection of knocked-out nucleon             - input 
* ksf    - spin projection of knocked-out nucleon                - input 
* pvec_f - three vector of knocked-out nucleon                   - input 
* ktm    - isospin projection of initial nucleon                 - input 
* ksm    - spin projection of initial  nucleon                   - input 
* pvec_m - three vector of initial nucleon                       - input 
*
* ktr    - isospin projection of rescattered nucleon             - input 
* ksr    - spin projection of rescattered  nucleon               - input 
* pvec_r - three vector of rescattered nucleon                   - input 
* kss    - spin projection of spectator  nucleon                 - input 
* jhe    - spin projection of 3He                                - input 
* ifl = 0 !1- pole case, 2-pvi, 0-both                           - input
* md  = 2 !method of integration for PVI 1,2,3,4, 2- Gadap       - input
*
*
*
****************************************************************************
      dimension pvec_m(3),pvec_r(3)
      wf1_re = 0.0
      wf1_im = 0.0

      wf1_re_r = 0.0
      wf1_im_r = 0.0 
      wf1_re_s = 0.0
      wf1_im_s = 0.0 

      if(ktm.eq.1)then !knocked out is proton
      irs = 22
      call A1nd_wf(q0,qv,ksf,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,
     &                             wf1_re_r,wf1_im_r,md,ifl,irs)
      irs = 21
      call A1nd_wf(q0,qv,ksf,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,
     &                             wf1_re_s,wf1_im_s,md,ifl,irs)
*      write(6,*)"ahairs"
                                            
      endif

      if(ktm.eq.-1)then !knocked out is neutron
*      write(6,*)q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,ifl
      irs = 121
      call A1nd_wf(q0,qv,ksf,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,
     &                             wf1_re_r,wf1_im_r,md,ifl,irs)
      irs = 122
      call A1nd_wf(q0,qv,ksf,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,
     &                             wf1_re_s,wf1_im_s,md,ifl,irs)
 
      endif
      wf1_re = wf1_re_r + wf1_re_s
      wf1_im = wf1_im_r + wf1_im_s
*     print *,")()()(",wf1_im
      return
      end




      subroutine A1_wf_chex_sum(q0,qv,ksf,ktmc,ksm,pvec_m,
     & ktr,ksr,pvec_r,kts,kss,jhe,wf1_re,wf1_im,ifl,md,ivr)
***************************************************************************
* Subroutine calculates the distorted due to single FSI wave function
* summed by the  number of spectator scatterrers.
*
* a1_re and a1_im - real and imaginary parts                  - output
*
* q0     - virtual photon energy                                 - input
* qv     - virtual photon momentum                               - input
* ktf    - isospin projection of knocked-out nucleon             - input 
* ksf    - spin projection of knocked-out nucleon                - input 
* pvec_f - three vector of knocked-out nucleon                   - input 
* ktm    - isospin projection of initial nucleon                 - input 
* ksm    - spin projection of initial  nucleon                   - input 
* pvec_m - three vector of initial nucleon                       - input 
*
* ktr    - isospin projection of rescattered nucleon             - input 
* ksr    - spin projection of rescattered  nucleon               - input 
* pvec_r - three vector of rescattered nucleon                   - input 
* kss    - spin projection of spectator  nucleon                 - input 
* jhe    - spin projection of 3He                                - input 
* ifl = 0 !1- pole case, 2-pvi, 0-both                           - input
* md  = 2 !method of integration for PVI 1,2,3,4, 2- Gadap       - input
*
*
*
****************************************************************************
      dimension pvec_m(3),pvec_r(3)
    
      wf1_re = 0.0
      wf1_im = 0.0

      wf1_re_r = 0.0
      wf1_im_r = 0.0 
      wf1_re_s = 0.0
      wf1_im_s = 0.0 

      if(ktmc.eq.1)then !knocked out is proton
      irs = 12   ! 1 interacts with 2
      call  A1_wf_chex(q0,qv,ksf,ktmc,ksm,pvec_m,ktr,ksr,pvec_r,
     &                    kts,kss,jhe,wf1_re_r,wf1_im_r,md,ifl,irs,ivr)
      irs = 13   ! 1 interacts with 3
      call  A1_wf_chex(q0,qv,ksf,ktmc,ksm,pvec_m,ktr,ksr,pvec_r,
     &                    kts,kss,jhe,wf1_re_s,wf1_im_s,md,ifl,irs,ivr)
*      write(6,*)"ahairs",ktr,kts,ksr,kss

      endif

      if(ktmc.eq.-1)then !knocked out is neutron
*      write(6,*)q0,qv,ktm,ksm,pvec_m,ktr,ksr,pvec_r,kts,kss,jhe,ifl
       if(ktr.eq.-1)then ! r particle is neutron
       irs = 12    ! chex happens with the second particle only
       call  A1_wf_chex(q0,qv,ksf,ktmc,ksm,pvec_m,ktr,ksr,pvec_r,
     &                   kts,kss,jhe,wf1_re_r,wf1_im_r,md,ifl,irs,ivr)
       wf1_re_s = 0.0
       wf1_im_s = 0.0
       elseif(kts.eq.-1)then
      irs = 13  ! chex happens with the third particle
      call  A1_wf_chex(q0,qv,ksf,ktmc,ksm,pvec_m,ktr,ksr,pvec_r,
     &                  kts,kss,jhe,wf1_re_s,wf1_im_s,md,ifl,irs,ivr)
      wf1_re_r = 0.0
      wf1_im_r = 0.0
      endif
*      write(6,*)"ahairrrrrr"

      endif

      wf1_re = wf1_re_r + wf1_re_s
      wf1_im = wf1_im_r + wf1_im_s

      return
      end



      subroutine A1_wf_chex_sum_ab(q0,qv,ksf,ktmc,ksm,pvec_m,
     & ktr,ksr,pvec_r,kts,kss,jhe,wf1_re,wf1_im,ifl,md,ivr,idn)
***************************************************************************
* Subroutine calculates the distorted due to single FSI wave function
* summed by the  number of spectator scatterrers.
*
* a1_re and a1_im - real and imaginary parts                  - output
*
* q0     - virtual photon energy                                 - input
* qv     - virtual photon momentum                               - input
* ktf    - isospin projection of knocked-out nucleon             - input 
* ksf    - spin projection of knocked-out nucleon                - input 
* pvec_f - three vector of knocked-out nucleon                   - input 
* ktm    - isospin projection of initial nucleon                 - input 
* ksm    - spin projection of initial  nucleon                   - input 
* pvec_m - three vector of initial nucleon                       - input 
*
* ktr    - isospin projection of rescattered nucleon             - input 
* ksr    - spin projection of rescattered  nucleon               - input 
* pvec_r - three vector of rescattered nucleon                   - input 
* kss    - spin projection of spectator  nucleon                 - input 
* jhe    - spin projection of 3He                                - input 
* ifl = 0 !1- pole case, 2-pvi, 0-both                           - input
* md  = 2 !method of integration for PVI 1,2,3,4, 2- Gadap       - input
*
*
*
****************************************************************************
      dimension pvec_m(3),pvec_r(3)
    
      wf1_re = 0.0
      wf1_im = 0.0

      wf1_re_rs = 0.0
      wf1_im_rs = 0.0 
*      print *,"idn",idn
      call  A1_wf_chex_ab(q0,qv,ksf,ktmc,ksm,pvec_m,ktr,ksr,pvec_r,
     &             kts,kss,jhe,wf1_re_rs,wf1_im_rs,md,ifl,irs,ivr,idn)


      wf1_re = wf1_re_rs 
      wf1_im = wf1_im_rs 
*      print *,wf1_re,wf1_im
      return
      end




       subroutine  A0s_wf_full 
     &  (kt1,ks1,p_m,kt2,ks2,p_r,kt3,ks3,kj,a0s_re,a0s_im,ifl)
******************************************************************
* inputs 
* ifl = 1 initializes the wave function, 2- resets the kj, 
*       3- as in A0s_wf initializes the amplitudes, but now said ampl.
* ifl = 0 calculates 
* kti - (i=1,2,3) isospins(1->proton,-1->neutron)
* ks1,ks2,ks3 - spin projection of 1,2,3 nucleon -1-> -1/2, 1-> 1/2
* kj - total spin  projection of he3 -1-> -1/2, 1-> 1/2
* p_m - missing momentum vector
* p_r - one of the recoil momentum vector 
* a0s_re, a0s_im - real and imaginary parts of the amiltitude_0
*/par/pi,pm,pmp,pmn,tm   : pi and masses(constants)
*******************************************************************
      external th23_a,th23_b,un23_re_full,un23_im_full
      dimension p_m(3),p_r(3),BB(10),pv1(3),pv2(3)
      common/par/pi,pm,pmp,pmn,tm,eb
      common/conf/it1,is1,it2,is2,it3,is3,ij
      common/missing/  p_m_x,  p_m_y,  p_m_z
      common/recoil_r/ p_r_x,  p_r_y,  p_r_z
      common/recoil_s/ p_s_x,  p_s_y,  p_s_z
      common/flag23/ifl23
      common/Bystr/ibys !new flag,when ibys = 1 reads said ampl.from data file
      ibys = 1

      a0s_re = 0.0
      a0s_im = 0.0
      if(ifl.eq.1)then
      call 
     & wf_he3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,kj,wfre,wfim,ifl)
      return
      elseif(ifl.eq.2)then
      call 
     & wf_he3_nb(kt1,ks1,pv1,kt2,ks2,pv2,kt3,ks3,kj,wfre,wfim,ifl)
      return
      elseif(ifl.eq.3)then
*       CALL CPU_TIME(time_begin)
       call Bystricky_dat(0.01,0.,-1,BB) ! initialization
*       CALL CPU_TIME(time_end)
*       PRINT*, 'Time of initialization was ', time_end - time_begin,' s'
*       print*, ' '  
      return
      endif

        it1    = kt1
        is1    = ks1
        it2    = kt2
        is2    = ks2
        it3    = kt3
        is3    = ks3
        ij     = kj

        p_m_x =  p_m(1)
        p_m_y =  p_m(2)
        p_m_z =  p_m(3)

        p_r_x =  p_r(1)
        p_r_y =  p_r(2)
        p_r_z =  p_r(3)

        p_s_x = -(p_m_x + p_r_x)  
        p_s_y = -(p_m_y + p_r_y) 
        p_s_z = -(p_m_z + p_r_z) 
     
      ifl23 = 2
      if(kt2.eq.1.and.kt3.eq.1) ifl23=1
*     ifl23 = 2 --> pn, 1 --> pp (for recoiled nucleon pair pair)
    
      eps23   = 0.05
      phi23_a = 0.0
      phi23_b = 2.0*pi
 
      call gadap2(phi23_a,phi23_b,th23_a,th23_b,un23_re_full,
     &                       eps23,sum_re) 

      eps23   = 0.05
      phi23_a = 0.0
      phi23_b = 2.0*pi
      call gadap2(phi23_a,phi23_b,th23_a,th23_b,un23_im_full,
     &                   eps23,sum_im) 

      fc = 1      
      a0s_re = fc*sum_re/(8.0*pi)
      a0s_im = fc*sum_im/(8.0*pi)
      return
      end

      function un23_re_full(phi23,th23)
      dimension pv1(3), pv2(3)
      common/par/pi,pm,pmp,pmn,tm,eb
      common/conf/mt1,ms1,mt2,ms2,mt3,ms3,mj
      common/missing/  p_m_x,  p_m_y,  p_m_z
      common/recoil_r/ p_r_x,  p_r_y,  p_r_z
      common/recoil_s/ p_s_x,  p_s_y,  p_s_z
      common/flag23/ifl
      un23_re_full = 0.0

      p_rs_x = (p_r_x - p_s_x)/2.0
      p_rs_y = (p_r_y - p_s_y)/2.0
      p_rs_z = (p_r_z - p_s_z)/2.0
      p_rs   = sqrt(p_rs_x**2 + p_rs_y**2 + p_rs_z**2)
      p_23   = p_rs
      p_23_x = p_23*sin(th23)*cos(phi23)
      p_23_y = p_23*sin(th23)*sin(phi23)
      p_23_z = p_23*cos(th23)

      p_2_x = p_23_x + (p_r_x + p_s_x)/2.0
      p_2_y = p_23_y + (p_r_y + p_s_y)/2.0
      p_2_z = p_23_z + (p_r_z + p_s_z)/2.0
      
      pv1(1) = p_m_x
      pv1(2) = p_m_y
      pv1(3) = p_m_z

      pv2(1) = p_2_x
      pv2(2) = p_2_y
      pv2(3) = p_2_z
*     ~~~~~~~~~~~~~~~~~~  Calculating s and t and u  ~~~~~~~~~~~~~~~
      p_2 = p_2_x**2 + p_2_y**2 + p_2_z**2
      E_2 = sqrt(p_2+pm**2)

      p_s = p_s_x**2 + p_s_y**2 + p_s_z**2
      E_s = sqrt(p_s+pm**2)

      p_r = p_r_x**2 + p_r_y**2 + p_r_z**2
      E_r = sqrt(p_r+pm**2)

        if(ifl.eq.1)then          ! pp case
        s = 2.*(pmp**2+E_s*E_r - p_s_x*p_r_x -p_s_y*p_r_y -p_s_z*p_r_z)
        t = 2.* pm**2 -
     &       2.*(E_2*E_r - p_2_x*p_r_x - p_2_y*p_r_y - p_2_z*p_r_z)
        elseif(ifl.eq.2)then      ! pn case (for a while the same as pp)
        s = 2.* pm**2+2.0*(E_r*E_s - 
     &                p_r_x*p_s_x -p_r_y*p_s_y -p_r_z*p_s_z) 
        t =  2.* pm**2 - 
     &       2.*(E_2*E_r - p_2_x*p_r_x - p_2_y*p_r_y - p_2_z*p_r_z)
        endif
      
*      u = 4.*pm**2 - s - t



*~~~~~~~~~~~~~~~~~~~~~~~~~  PP ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if(ifl.eq.1) then 
          ins = -22 ! pp scattering without Coulumb effects.
           
          
          sum = 0.0
          do is2=1,-1,-2
          do is3=1,-1,-2
**             is2 = ms2
**             is3 = ms3
            call
     &      wf_he3_nb(mt1,ms1,pv1,mt2,is2,pv2,mt3,is3,mj,wfre,wfim,0)
              ! f_pp is a Feynman f in GeV-2
            call  f_NN_diff_said(is2,is3,ms2,ms3, 
     &                           s,t,f_pp_re,f_pp_im,ins) 
 
            f_til_re =  -(s-4.*pm**2)/8./pi*f_pp_im 
            f_til_im =   (s-4.*pm**2)/8./pi*f_pp_re

            tms  = 0.5 ! this accounts for the two proton identities

           sum = sum + tms*(wfre*f_til_re - wfim*f_til_im)*sin(th23)
          enddo
          enddo
*~~~~~~~~~~~~~~~~~~~~~~~~ END PP ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
          
*~~~~~~~~~~~~~~~~~~~~~~~~~  PN ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        elseif(ifl.eq.2) then ! SAID ELAS_PN and CHEX_PN
          ins = 21

          sum = 0.0
*          do is2=1,-1,-2
*          do is3=1,-1,-2
          do is2=-1,1,2
          do is3=-1,1,2

*~~~~~~~~~~                 ELASTIC         ~~~~~~~~~~~~~~~~~~~~~~~~~~
            call  wf_he3_nb
     &            (mt1,ms1,pv1,mt2,is2,pv2,mt3,is3,mj,wfre_el,wfim_el,0)

            call  f_NN_diff_said(is2,is3,ms2,ms3,
     &                           s,t,f_pn_el_re,f_pn_el_im,ins) 
            f_til_el_re =  -(s-4.*pm**2)/8./pi*f_pn_el_im
            f_til_el_im =   (s-4.*pm**2)/8./pi*f_pn_el_re

*~~~~~~~~~~             CHARGE EXCHANGE         ~~~~~~~~~~~~~~~~~~~~~~~
* Here we dont need to include the charge exchange explicitely, since
* elastic contribution at \theta_cm>90 accounts for it 
* automaticaly
***********************************************************************
*            call wf_he3_nb
*     &      (mt1,ms1,pv1,-mt2,is2,pv2,-mt3,is3,mj,wfre_chex,wfim_chex,0)
*
*            call  f_NN_diff_said(is3,is2,ms2,ms3,
*     &                         s,u,f_pn_chex_re,f_pn_chex_im,ins) 
* 
*
*    	     call f_NN_diff_said_chex(is2,is3,ms2,ms3,
*     &                           s,t,f_pn_chex_re,f_pn_chex_im)
*
*
*            sn = 1.0
*            f_til_chex_re =  -sn*(s-4.*pm**2)/8./pi*f_pn_chex_im
*            f_til_chex_im =   sn*(s-4.*pm**2)/8./pi*f_pn_chex_re
*            tms = 0.5
*            sum = sum + tms*sin(th23)
*     &                *(wfre_el*f_til_el_re + wfre_chex*f_til_chex_re 
*     &                - wfim_el*f_til_el_im - wfim_chex*f_til_chex_im)
*************************************************************************
            sum = sum + 
     &            sin(th23)*(wfre_el*f_til_el_re - wfim_el*f_til_el_im)

          enddo
          enddo
*~~~~~~~~~~~~~~~~~~~~~~~~ END PN ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        endif

        un23_re_full = sum
  
      return
      end


      function un23_im_full(phi23,th23)
      dimension pv1(3), pv2(3)
      common/par/pi,pm,pmp,pmn,tm,eb
      common/conf/mt1,ms1,mt2,ms2,mt3,ms3,mj
      common/missing/  p_m_x,  p_m_y,  p_m_z
      common/recoil_r/ p_r_x,  p_r_y,  p_r_z
      common/recoil_s/ p_s_x,  p_s_y,  p_s_z
      common/flag23/ifl
      un23_im_full = 0.0 

      p_rs_x = (p_r_x - p_s_x)/2.0
      p_rs_y = (p_r_y - p_s_y)/2.0
      p_rs_z = (p_r_z - p_s_z)/2.0
      p_rs   = sqrt(p_rs_x**2 + p_rs_y**2 + p_rs_z**2)
      p_23   = p_rs
      p_23_x = p_23*sin(th23)*cos(phi23)
      p_23_y = p_23*sin(th23)*sin(phi23)
      p_23_z = p_23*cos(th23)

      p_2_x = p_23_x + (p_r_x + p_s_x)/2.0
      p_2_y = p_23_y + (p_r_y + p_s_y)/2.0
      p_2_z = p_23_z + (p_r_z + p_s_z)/2.0
      
      pv1(1) = p_m_x
      pv1(2) = p_m_y
      pv1(3) = p_m_z

      pv2(1) = p_2_x
      pv2(2) = p_2_y
      pv2(3) = p_2_z

*     ~~~~~~~~~~~~~~~~~~  Calculating s and t and u  ~~~~~~~~~~~~~~~
      p_2 = p_2_x**2 + p_2_y**2 + p_2_z**2
      E_2 = sqrt(p_2+pm**2)

      p_s = p_s_x**2 + p_s_y**2 + p_s_z**2
      E_s = sqrt(p_s+pm**2)

      p_r = p_r_x**2 + p_r_y**2 + p_r_z**2
      E_r = sqrt(p_r+pm**2)

        if(ifl.eq.1)then          ! pp case
        s = 2.*(pmp**2+E_s*E_r - p_s_x*p_r_x -p_s_y*p_r_y -p_s_z*p_r_z)
        t = 2.* pm**2 -
     &       2.*(E_2*E_r - p_2_x*p_r_x - p_2_y*p_r_y - p_2_z*p_r_z)
        elseif(ifl.eq.2)then     ! pn case
        s = 2.* pm**2+2.0*(E_r*E_s - 
     &                p_r_x*p_s_x -p_r_y*p_s_y -p_r_z*p_s_z) 
        t =  2.* pm**2 - 
     &       2.*(E_2*E_r - p_2_x*p_r_x - p_2_y*p_r_y - p_2_z*p_r_z)
        endif
      
*      u = 4.*pm**2 - s - t

*~~~~~~~~~~~~~~~~~~~~~~~~~  PP ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        if(ifl.eq.1) then 
          ins = -22  ! with no Coulomb effects
          sum = 0.0                            
          do is2=1,-1,-2
          do is3=1,-1,-2

*             is2 =  ms2
*             is3 =  ms3

            call
     &      wf_he3_nb(mt1,ms1,pv1,mt2,is2,pv2,mt3,is3,mj,wfre,wfim,0)
              ! f_pp_re(im) is a Feynman f in GeV-2

            call  f_NN_diff_said(is2,is3,ms2,ms3, 
     &                           s,t,f_pp_re,f_pp_im,ins) 
 
            f_til_re =  -(s-4.*pm**2)/8./pi*f_pp_im
            f_til_im =  (s-4.*pm**2)/8./pi*f_pp_re

            tms = 0.5 ! this accounts for the two proton identities
           sum = sum + tms*(wfre*f_til_im + wfim*f_til_re)*sin(th23)

          enddo
          enddo
*~~~~~~~~~~~~~~~~~~~~~~~~ END PP ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*~~~~~~~~~~~~~~~~~~~~~~~~~  PN ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        elseif(ifl.eq.2) then ! SAID ELAS_PN and CHEX_PN
          ins = 21
          
          sum = 0.0
*          do is2=1,-1,-2
*          do is3=1,-1,-2
          do is2=-1,1,2
          do is3=-1,1,2

*~~~~~~~~~~                 ELASTIC         ~~~~~~~~~~~~~~~~~~~~~~~~~~
            call wf_he3_nb(mt1,ms1,pv1,mt2,is2,pv2,mt3,is3,mj,
     &                                          wfre_el,wfim_el,0)

            call  f_NN_diff_said(is2,is3,ms2,ms3,
     &                           s,t,f_pn_el_re,f_pn_el_im,ins) 
            f_til_el_re =  -(s-4.*pm**2)/8./pi*f_pn_el_im
            f_til_el_im =   (s-4.*pm**2)/8./pi*f_pn_el_re

*~~~~~~~~~~             CHARGE EXCHANGE         ~~~~~~~~~~~~~~~~~~~~~~~
* Here we dont need to include the charge exchange explicitely, since
* elastic contribution at \theta_cm>90 accounts for it 
* automaticaly
*
*            call wf_he3_nb(mt1,ms1,pv1,-mt2,is2,pv2,-mt3,is3,mj,
*     &                                        wfre_chex,wfim_chex,0)
*
*            call  f_NN_diff_said(is3,is2,ms2,ms3,
*     &                         s,u,f_pn_chex_re,f_pn_chex_im,ins) 
*
*    	     call f_NN_diff_said_chex(is2,is3,ms2,ms3,
*     &                           s,t,f_pn_chex_re,f_pn_chex_im)
*
*
*            sn = 1.0
*            f_til_chex_re =   -sn*(s-4.*pm**2)/8./pi*f_pn_chex_im
*            f_til_chex_im =    sn*(s-4.*pm**2)/8./pi*f_pn_chex_re
*----------------------------------------------------------------------
*            tms = 0.5
*            sum = sum + tms*sin(th23)
*     &                *(wfre_el*f_til_el_im + wfre_chex*f_til_chex_im 
*     &                + wfim_el*f_til_el_re + wfim_chex*f_til_chex_re)
            sum = sum + 
     &            sin(th23)*(wfre_el*f_til_el_im + wfim_el*f_til_el_re) 

          enddo
          enddo
*~~~~~~~~~~~~~~~~~~~~~~~~ END PN ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        endif

        un23_im_full = sum
  
      return
      end

        function th23_a(phi)
        th23_a = 0.0
        return
        end

        function th23_b(phi)
        th23_b = acos(-1.0)
        return
        end


 	subroutine polar(p_x,p_y,p_z,p,th,phi)
        p = sqrt(p_x**2 + p_y**2 + p_z**2)
        th = 0.0
        phi = 0.0
        if(p.eq.0.0)return
	arg = p_z/p
	if(arg.gt. 1.0)arg =  1.0
	if(arg.lt.-1.0)arg = -1.0
	th  = acos(arg)
	sn_th = sin(th)
	phi = 0.0
        if(p_y.eq.0.0.and.p_x.lt.0.0)then
        phi = acos(-1.0)
        return
        elseif(p_y.eq.0.0.and.p_x.gt.0.0)then
        phi = 0.0
        return
        endif
	if(sn_th.ne.0.0)then
	arg = p_x/p/sn_th
	if(arg.gt. 1.0)arg =  1.0
	if(arg.lt.-1.0)arg = -1.0
	phi = acos(arg)
	if((p_y/sn_th).lt.0.0)phi = 2.0*acos(-1.0) - phi
	endif
	return
	end



	SUBROUTINE GO_BACK(PP,THP,PHIP,TH,PHI)
        if(pp.eq.0.0)return
	PPX = PP*SIN(THP)*COS(PHIP)
	PPY = PP*SIN(THP)*SIN(PHIP)
	PPZ = PP*COS(THP)
	PX  = (PPX*COS(TH) + PPZ*SIN(TH))*COS(PHI) - PPY*SIN(PHI)
	PY  = (PPX*COS(TH) + PPZ*SIN(TH))*SIN(PHI) + PPY*COS(PHI)
	PZ  =   PPZ*COS(TH) - PPX*SIN(TH)
	PP  = SQRT(PX**2 + PY**2 + PZ**2)
	thp = 0.0
	phip = 0.0
	if(pp.ne.0.0)then
	ARG = PZ/PP
	IF(ARG.GT.1.0 ) ARG =  1.0
	IF(ARG.LT.-1.0) ARG = -1.0
	THP = ACOS(ARG)
	PHIP = 0.0
	IF(SIN(THP).NE.0.0)THEN
	ARG = PX/PP/SIN(THP)
	IF(ARG.GT.1.0 ) ARG =  1.0
	IF(ARG.LT.-1.0) ARG = -1.0
	PHIP = ACOS(ARG)
	IF(PY/SIN(THP).LT.0.0)PHIP = 6.2831 - PHIP
	ENDIF 
	endif
	RETURN
	END
