      subroutine j_elm_pol_db(nuc,q2,q,p2,ipl2,s2,p1,ipl1,s1,j_elm)
*************************************************************************************
* Calculates electromagnetic current of nucleon and electron
*  <s2,p2|J^\mu|p1,s1>  = \bar u(p_2,s_2)\Gamma^\mu u(p_1,s_1) - j_elm_re/im(s2,s1), where
*  1,1 = u2,u1, 1,2=u2,d1,  2,1=d2,u1, 2,2=d2,d1
*
* here \Gamma_\mu = \gamma_mu F1 + i/(2M) F2 q^\nu \sigma_{\mu\nu} - for nucleon
*      \Gamma_\mu = \gamma_mu - for electron
*
* nuc - (0) initializes the \gamma and \sigma matrices
* nuc - (1) - proton (-1) neutron, 3 - electron
* q2 - Q2
* q  - q(3)  three momentum of the virtual photon
* p1 - p1(3) three momentum of the initial nucleon/electron
* p2 - p2(3) three momentum of the final nucleon/electron 
* 
* s1 - s1(3) polarization vector for incoming lepton
* s2 - s2(3) polarization vector for outgoing lepton
*
* j_elm(0:3,2f,2i) - complex out - returns re and im part of the current
*               0:3 are mu components,  2f - final   nucleon's spin 1-up 2-down
*                                       2i - initial nucleon's spin 1-up 2-down
* ipl1 = 1/0 - incoming fermion is polarized/not polarized
* ipl2 = 1/0 - outgoing fermion is polarized/not polarized
*
*! common/parm/pi,pm,dm,em should be provided 
*
*
*
* Misak Sargsian 
* March 2007,FIU, Miami
* 
* Polarization projection part is aded as well as electron current
* July 2007, FIU, Miami
*
****************************************************************************************
      implicit none
      complex*16 , dimension(0:3,4,4):: bgamma
      common/gammamatices/gamma,sigma,unitm,gamma5,epsl,bigsigma
      complex*16, dimension(0:3,4,4)         :: gamma
      complex*16, dimension(0:3,0:3,4,4)     :: sigma
      complex*16, dimension(4,4)             :: unitm,gamma5
      real*8,     dimension(0:3,0:3)         :: gmetr
      real*8,     dimension(0:3,0:3,0:3,0:3) :: epsl
      complex*16, dimension(3,4,4)           :: bigsigma
 
      complex*16, dimension(4):: u1,uc1,u2,uc2,u2bar,ul0,ulx,uly,ulz
      complex*16, dimension(0:3,4):: ucgamma,ucmat,u2bar_bgamma
      real*8,     dimension(3) :: q,p2,p1,s2,s1
      complex*16, dimension(0:3,2,2):: j_elm

      complex*16 c,cm,j0,jx,jy,jz
      real*8 pi,pm,dm,em,m
      real*8 p1x,p1y,p1z,e1,  p2x,p2y,p2z,e2 , qx,qy,qz,q0,q2 
      integer nuc,is1,isp1,is2,isp2,ipl2,ipl1
      common/parm/pi,pm,dm,em
      common/zeroc/zeroc
      complex*16 zeroc
                  m = pm
      if(nuc.eq.3)m = em

      j_elm = dcmplx(0.0d+0,0.0d+0)
      if(nuc.eq.0)then
      call gammamatrices  ! initializes gamma and sigma matrices
      zeroc = dcmplx(0.0d+0,0.0d+0)
      return
      endif

      p1x = p1(1)
      p1y = p1(2)
      p1z = p1(3)
      e1  = dsqrt(m**2 + p1x**2 + p1y**2 + p1z**2)

      p2x = p2(1)
      p2y = p2(2)
      p2z = p2(3)
      e2  = dsqrt(m**2 + p2x**2 + p2y**2 + p2z**2)

      
      qx = q(1)
      qy = q(2)
      qz = q(3)
      q0 = dsqrt(qx**2+qy**2+qz**2-q2)



      do is1 = 1,2
                     isp1 =   1
         if(is1.eq.2)isp1 =  -1
      do is2 = 1,2
                     isp2 =  1
         if(is2.eq.2)isp2 = -1
      
*******************************************************
*      Spinor of initial nucleon
*******************************************************
         u1  = zeroc !cmplx(0.0d+0,0.0d+0)
         uc1 = zeroc !cmplx(0.0d+0,0.0d+0) 
      if(ipl1.eq.1)then
*      call uspol(m,p1x,p1y,p1z,e1,s1,isp1,u1,uc1)
      call uspol_v2(m,p1x,p1y,p1z,e1,s1,isp1,u1,uc1)
*      call    us(m,p1x,p1y,p1z,e1,isp1,u1,uc1)
      else
      call    us(m,p1x,p1y,p1z,e1,isp1,u1,uc1)
      endif
*      print *,"u1",u1
*******************************************************
*      Spinor of final nucleon
*******************************************************
         u2  = zeroc !cmplx(0.0d+0,0.0d+0)
         uc2 = zeroc !cmplx(0.0d+0,0.0d+0) 
      if(ipl2.eq.1)then
      call uspol_v2(m,p2x,p2y,p2z,e2,s2,isp2,u2,uc2)
*      call uspol(m,p2x,p2y,p2z,e2,s2,isp2,u2,uc2)
      else
      call    us(m,p2x,p2y,p2z,e2,isp2,u2,uc2)
      endif
*      a =   matmul(gamma(1,1:4,1:4),gamma(1,1:4,1:4))
*      b =   matmul(gamma(1,1:4,1:4),gamma(1,1:4,1:4))
*      call vec_vec_sprod(uc1,u1,c)
********************************************************
*     Calculating u2bar
********************************************************
      call vec_mat_prod(uc2,gamma,ucgamma)  
      u2bar = ucgamma(0,1:4)

******************************************************************
*      Calculating nucleon vertex
******************************************************************
      call vertex(nuc,q0,qx,qy,qz,bgamma)

*      print *, "bgamma",bgamma(0,1:4,1:4)
*****************************************************************
*      Calculating \bar u2 \Gamma part
*****************************************************************
      call vec_mat_prod(u2bar,bgamma,u2bar_bgamma)  
      
      ul0 = u2bar_bgamma(0,1:4)
      ulx = u2bar_bgamma(1,1:4)
      uly = u2bar_bgamma(2,1:4)
      ulz = u2bar_bgamma(3,1:4)

*      call vec_vec_sprod(ul0,u2,c)
*      print *, "c",c,2.*e2
*      call vec_vec_sprod(uc2,u1,c)
*      print *, "c",c

****************************************************************
*      Completing the calculation of current components
****************************************************************
      call vec_vec_sprod(ul0,u1,j0)
      call vec_vec_sprod(ulx,u1,jx)
      call vec_vec_sprod(uly,u1,jy)
      call vec_vec_sprod(ulz,u1,jz)

      j_elm(0,is2,is1) = j0
      j_elm(1,is2,is1) = jx
      j_elm(2,is2,is1) = jy
      j_elm(3,is2,is1) = jz

      enddo
      enddo

      return
      end

      subroutine vec_vec_sprod(a,b,c)
      complex*16, dimension(4) :: a, b
      complex*16 c
      c = a(1)*b(1) + a(2)*b(2) + a(3)*b(3)+a(4)*b(4)
      return
      end

      subroutine vec_mat_prod(a,b,pr)
******************************************************
*  a(4)   - vector
*  b(0:3,4,4) - matrix
*  pr(0:3,4) = a*b 
******************************************************
      complex*16, dimension(4)       :: a
      complex*16, dimension(0:3,4,4) :: b
      complex*16, dimension(0:3,4)   :: pr
      common/zeroc/zeroc
      complex*16 zeroc

      pr = zeroc !cmplx(0.0d+0,0.0d+0)
      do mu = 0,3
      do j = 1,4
      do i = 1,4
      pr(mu,j)  = pr(mu,j) + a(i)*b(mu,i,j)
      enddo
      enddo
      enddo
*      print *,"*",a
*      print *,"***", pr(0,1:4)
      return
      end


      subroutine vertex(nuc,q0,qx,qy,qz,bgamma)
***************************************************************************
*  This subroutine calculates the elastic electromagnetc vertex of nucleon
*   \Gamma_\mu = \gamma_mu F1 + i/(2M) F2 q^\nu \sigma_{\mu\nu}
* 
*   nuc = 1 proton, -1 neutron
*   q0,qx,qy,qz - energy and momentum components of virtual photon
*   bgamma(0:3,4,4)  \gamma_m  4x4 matrix for \mu = 0-3
*
*
* Misak Sargsian
* March 2007, FIU, Miami
******************************************************************************
      implicit none
      common/parm/pi,pm,dm,em
*      common/gammamatices/gamma,sigma
      common/gammamatices/gamma,sigma,unitm,gamma5,epsl,bigsigma
      complex*16, dimension(0:3,4,4):: gamma,bgamma,sigmaq
      complex*16, dimension(0:3,0:3,4,4) :: sigma
      complex*16, dimension(4,4)            :: unitm,gamma5
      real*8,     dimension(0:3,0:3,0:3,0:3):: epsl
      complex*16, dimension(3,4,4)           :: bigsigma

      complex*16 fdir_1,fdir_2
      real*8 pi,pm,dm,em
      real*8 q0,qx,qy,qz,qv2,q2,F1,F2
      integer nuc,mu,j,i
      if(nuc.eq.3)then
      bgamma = gamma
      return
      endif

      qv2 = qx**2 + qy**2 + qz**2
      q2  = qv2 - q0**2
      do mu = 0,3
      do j = 1,4
      do i = 1,4
      sigmaq(mu,i,j) = q0*sigma(mu,0,i,j) - qx*sigma(mu,1,i,j)
     &                -qy*sigma(mu,2,i,j) - qz*sigma(mu,3,i,j)

      enddo
      enddo
      enddo
     
      fdir_1 = cmplx(F1(nuc,q2), 0.0d+0)
      fdir_2 = cmplx(0.0d+0    , 1.0d+0/(2.0d+0*pm)*F2(nuc,q2))

      bgamma =fdir_1*gamma + fdir_2*sigmaq
*      print *,"sigma",fdir_2
      return
      end





     
**********************************************************
*  Dirac Nucleon Form Factors for Nucleon
**********************************************************
      function F1(nuc,q2)
      implicit none
      integer nuc
      real*8 q2,F1,F1p,F1n
*      change
      if(nuc.eq. 1)F1 = F1p(q2)
      if(nuc.eq.-1)F1 = F1n(q2)
      return
      end

      function F2(nuc,q2)
      implicit none
      integer nuc
      real*8 q2,F2,F2p,F2n
*       change
      if(nuc.eq. 1)F2 = F2p(q2)
      if(nuc.eq.-1)F2 = F2n(q2)
      return
      end


      function F1p(Q2)
      implicit none
      integer nuc
      real*8 pi,pm,dm,em,q2
      real*8 tau,GEp,GMp,F1p
      common/parm/pi,pm,dm,em
      call proton_formfactor(Q2,GEp,GMp)
      tau = Q2/4.d+0/pm**2             
      F1p = (tau*GMp+GEp)/(1.d+0 + tau)                                        
      return                                                            
      end                                                            
C     ----------------------------------------                          
      function F2p(Q2)                                                  
      implicit none
      integer nuc
      real*8 pi,pm,dm,em,q2
      real*8 tau,GEp,GMp,F2p
      common/parm/pi,pm,dm,em
      call proton_formfactor(Q2,GEp,GMp)
      tau = Q2/4.d+0/pm**2                                                   
      F2p = (GMp-GEp)/(1.d+0+tau)    
      return                                             
      end                                                            
C     ---------------------------------------------                     
      function F1n(Q2)      
      implicit none
      integer nuc
      real*8 pi,pm,dm,em,q2
      real*8 tau,GEn,GMn,F1n                                            
      common/parm/pi,pm,dm,em
      call neutron_formfactor(Q2,GEn,GMn)
      tau = Q2/4.d+0/pm**2                                         
      F1n = (tau*GMn+GEn)/(1.d+0+tau)                                        
      return                                                            
      end                                                            
C     ---------------------------------------------                     
      function F2n(Q2)                                                  
      implicit none
      integer nuc
      real*8 pi,pm,dm,em,q2
      real*8 tau,GEn,GMn,F2n                                            
      common/parm/pi,pm,dm,em
      call neutron_formfactor(Q2,GEn,GMn)
      tau = Q2/4.d+0/pm**2                                                   
      F2n = (GMn-GEn)/(1.d+0+tau)                                            
      return                                                            
      end                                                            
C     -------------------------------------                             
*******************************************************
*   Charge and Magnetic Form Factor Parameterizations
*   Standard parameterization (one I used before)
*******************************************************
      subroutine proton_formfactor(q2,GEp,GMp)
      implicit none
      real*8 pi,pm,dm,em,q2
      real*8 GEp,GMp,G                                            
      common/parm/pi,pm,dm,em
      GEp = G(Q2)                                                         
      GMp = G(Q2)*2.79d+0                                                    
      return
      end
      subroutine neutron_formfactor(q2,GEn,GMn)
      implicit none
      real*8 pi,pm,dm,em,q2
      real*8 tau,GEn,GMn,G                                            
      common/parm/pi,pm,dm,em
      tau = Q2/4.d+0/pm**2               
      GEn =  1.91d+0*G(Q2)*tau/(1.d+0 + 5.6d+0*tau)                            
      GMn = -1.91d+0*G(Q2)                     
      return
      end
      function G(Q2)
      real *8 q2,G
      G=1./(1.+Q2/0.71d+0)**2                                               
      return                                                            
      end                                                            


      subroutine uspol(m,px,py,pz,e,s,isp,up,upc)
***************************************************************
*   This subroutine calculates Dirac Spinor for fermions with polarization spn
*   m - mass
*   px,py,pz,e - three components of momenta and energy
*   s(3) - spin three vector
*   isp - 1 spin - up  -1 spin down
*
*   up   - spinor
*   upc  - complex conjugate spinor
*
*
*
* Misak Sargsian
* July 2007, FIU, Miami
*
***************************************************************
      implicit none
      common/gammamatices/gamma,sigma,unitm,gamma5,epsl,bigsigma
      complex*16, dimension(0:3,4,4)        :: gamma
      complex*16, dimension(0:3,0:3,4,4)    :: sigma
      complex*16, dimension(4,4)            :: unitm,gamma5
      real*8,     dimension(0:3,0:3)        :: gmetr
      real*8,     dimension(0:3,0:3,0:3,0:3):: epsl
      complex*16, dimension(3,4,4)           :: bigsigma
      complex*16, dimension(4,4)             :: s4_hat,spin_pro

      real*8,       dimension(3)            :: s
      real*8,       dimension(0:3)          :: s4

      complex*16, dimension(4):: u,uc
      complex*16, dimension(4):: up,upc


      real*8 m,px,py,pz,e
      real*8 s_dot_p
      integer isp,i,j
      common/zeroc/zeroc
      complex*16 zeroc

*********************************************************
*   Constructing spin four vector
*********************************************************
      
      s_dot_p = s(1)*px + s(2)*py + s(3)*pz

      s4(0) = s_dot_p/m
      s4(1) = s(1) + s_dot_p*px/(m*(e+m))
      s4(2) = s(2) + s_dot_p*py/(m*(e+m))
      s4(3) = s(3) + s_dot_p*pz/(m*(e+m))
      s4_hat = zeroc !cmplx(0.0,0.0)
      s4_hat(1:4,1:4) = gamma(0,1:4,1:4)*s4(0) - 
     &                  gamma(1,1:4,1:4)*s4(1) - 
     &                  gamma(2,1:4,1:4)*s4(2) - 
     &                  gamma(3,1:4,1:4)*s4(3)  
      spin_pro(1:4,1:4) = 0.5d+0*(unitm(1:4,1:4) + 
     &                   matmul(gamma5(1:4,1:4),s4_hat(1:4,1:4)))


      call us(m,px,py,pz,e,isp,u,uc)
      
      up  = zeroc !cmplx(0.0,0.0)
      upc = zeroc
      do i = 1,4
      do j = 1,4
      up(i) = up(i) + spin_pro(i,j)*u(j)
      enddo 
      enddo
      upc(1:4) = conjg(up(1:4))
      return
      end



      subroutine uspol_v2(m,px,py,pz,e,s,isp,ups,upsc)
      implicit none
      complex*16, dimension(4)   :: ups, upsc
      real*8,    dimension(3)   :: s
      real*8,    dimension(0:3) :: s4
      real*8 m,px,py,pz,e, s_dot_p
      complex*16 p_pl,p_mn
      complex*16 s_0,s_pl,s_mn,s_z
      integer isp
      common/zeroc/zeroc
      complex*16 zeroc
      real *8 eplm,kes

      p_pl = dcmplx(px,py)
      p_mn = dcmplx(px,-py)
      e = dsqrt(m**2 + px**2 + py**2 + pz**2)
      eplm = e+m

      s_dot_p = s(1)*px + s(2)*py + s(3)*pz

      s4(0) = s_dot_p/m
      s4(1) = s(1) + s_dot_p/(m*(e+m)) * px
      s4(2) = s(2) + s_dot_p/(m*(e+m)) * py
      s4(3) = s(3) + s_dot_p/(m*(e+m)) * pz

      s_0   = dcmplx(s4(0),0.0d+0)
      s_pl  = dcmplx(s4(1),s4(2))
      s_mn  = dcmplx(s4(1),-s4(2))
      s_z   = dcmplx(s4(3),0.0d+0)
      kes = 0.5d+0
      if(isp.eq.1)then
      ups(1)=kes*dsqrt(eplm)*(1.0d+0  + s_z - s_0*pz/eplm)
      ups(2)=kes*dsqrt(eplm)*(s_pl - s_0*p_pl/eplm)
      ups(3)=kes*dsqrt(eplm)*(s_0 + (1.0d+0-s_z)*pz/eplm-s_mn*p_pl/eplm)
      ups(4)=kes*dsqrt(eplm)*(-s_pl*pz/eplm + (1.0d+0 + s_z)*p_pl/eplm)

      upsc(1)=kes*dsqrt(eplm)*(1.0d+0  + s_z - s_0*pz/eplm)
      upsc(2)=kes*dsqrt(eplm)*(s_mn - s_0*p_mn/eplm)
      upsc(3)=kes*dsqrt(eplm)*(s_0 +(1.0d+0-s_z)*pz/eplm-s_pl*p_mn/eplm)
      upsc(4)=kes*dsqrt(eplm)*(-s_mn*pz/eplm + (1.0d+0+s_z)*p_mn/eplm)


      elseif(isp.eq.-1.0)then
      ups(1)=kes*dsqrt(eplm)*(s_mn - s_0*p_mn/eplm)
      ups(2)=kes*dsqrt(eplm)*(1.0d+0 - s_z + s_0*pz/eplm)
      ups(3)=kes*dsqrt(eplm)*((1.0d+0-s_z)*p_mn/eplm + s_mn*pz/eplm)
      ups(4)=kes*dsqrt(eplm)*(s_0 - s_pl*p_mn/eplm-(1.0d+0+s_z)*pz/eplm)

      upsc(1)=kes*dsqrt(eplm)*(s_pl - s_0*p_pl/eplm)
      upsc(2)=kes*dsqrt(eplm)*(1.0d+0 - s_z + s_0*pz/eplm)
      upsc(3)=kes*dsqrt(eplm)*((1.0d+0-s_z)*p_pl/eplm + s_pl*pz/eplm)
      upsc(4)=kes*dsqrt(eplm)*(s_0 -s_mn*p_pl/eplm-(1.0d+0+s_z)*pz/eplm)
      endif

      return
      end





      subroutine us(m,px,py,pz,e,isp,u,uc)
***************************************************************
*   This subroutine calculates Dirac Spinor for unpolarized fermions
*   m - mass
*   px,py,pz,e - three components of momenta and energy
*   isp - 1 spin - up  -1 spin down
*
*   u   - spinor
*   uc  - complex conjugate spinor
*
*
*
* Misak Sargsian
* March 2007, FIU, Miami
*
***************************************************************
      implicit none
      complex*16, dimension(4):: u,uc
      real*8 m,px,py,pz,e,fct
      integer isp
      fct = dsqrt(e+m)
      if(isp.eq.1)then
      u(1) = fct*dcmplx(1.0d+0,0.0d+0)
      u(2) =     dcmplx(0.0d+0,0.0d+0)
      u(3) = fct/(e+m)*dcmplx(pz,0.d+0)
      u(4) = fct/(e+m)*dcmplx(px,py)

      uc(1) = fct*dcmplx(1.0d+0,0.0d+0)
      uc(2) =     dcmplx(0.0d+0,0.0d+0)
      uc(3) = fct/(e+m)*dcmplx(pz,0d+0)
      uc(4) = fct/(e+m)*dcmplx(px,-py)

      elseif(isp.eq.-1)then
      u(1) =     dcmplx(0.0d+0,0.0d+0)
      u(2) = fct*dcmplx(1.0d+0,0.0d+0)
      u(3) = fct/(e+m)*dcmplx(px,-py)
      u(4) = fct/(e+m)*dcmplx(-pz,0.0d+0)

      uc(1) =     dcmplx(0.0d+0,0.0d+0)
      uc(2) = fct*dcmplx(1.0d+0,0.0d+0)
      uc(3) = fct/(e+m)*dcmplx(px,py)
      uc(4) = fct/(e+m)*dcmplx(-pz,0.0d+0)
      endif
      return
      end


      


 



      

      subroutine gammamatrices
*********************************************************************************
*     This subroutine calculates Dirac gamma and sigma 
*     matrices.
*
*    sigma_{\mu,\nu} = {1\over 2}[\gamma_\mu\gamma_nu - \gamma_\nu\gamma_\mu}
*    gamma(0:3,4,4) 0:3 - are the values of \mu
*    (4,4) components are such
*     a(4,4)
*     a11 a12 a13 a14
*     a21 a22 a23 a24
*     a31 a32 a33 a34
*     a41 a42 a43 a44
*
*   March 2007, FIU, Miami
*********************************************************************************
      common/gammamatices/gamma,sigma,unitm,gamma5,epsl,bigsigma
      complex*16, dimension(0:3,4,4)         :: gamma
      complex*16, dimension(0:3,0:3,4,4)     :: sigma
      complex*16, dimension(4,4)             :: unitm,gamma5
      real*8,     dimension(0:3,0:3)         :: gmetr
      real*8,     dimension(0:3,0:3,0:3,0:3) :: epsl
      complex*16, dimension(3,4,4)           :: bigsigma
      complex*16 fct
***********************************************
*  gamma_0
***********************************************
      gamma(0,1,1) = dcmplx(1.,0.) 
      gamma(0,1,2) = dcmplx(0.,0.)
      gamma(0,1,3) = dcmplx(0.,0.)
      gamma(0,1,4) = dcmplx(0.,0.)

      gamma(0,2,1) = dcmplx(0.,0.) 
      gamma(0,2,2) = dcmplx(1.,0.)
      gamma(0,2,3) = dcmplx(0.,0.)
      gamma(0,2,4) = dcmplx(0.,0.)

      gamma(0,3,1) = dcmplx(0.,0.) 
      gamma(0,3,2) = dcmplx(0.,0.)
      gamma(0,3,3) = dcmplx(-1.,0.)
      gamma(0,3,4) = dcmplx(0.,0.)

      gamma(0,4,1) = dcmplx(0.,0.) 
      gamma(0,4,2) = dcmplx(0.,0.)
      gamma(0,4,3) = dcmplx(0.,0.)
      gamma(0,4,4) = dcmplx(-1.,0.)
***********************************************
*  gamma_x
***********************************************
      gamma(1,1,1) = dcmplx(0.,0.) 
      gamma(1,1,2) = dcmplx(0.,0.)
      gamma(1,1,3) = dcmplx(0.,0.)
      gamma(1,1,4) = dcmplx(1.,0.)

      gamma(1,2,1) = dcmplx(0.,0.) 
      gamma(1,2,2) = dcmplx(0.,0.)
      gamma(1,2,3) = dcmplx(1.,0.)
      gamma(1,2,4) = dcmplx(0.,0.)

      gamma(1,3,1) = dcmplx(0.,0.) 
      gamma(1,3,2) = dcmplx(-1.,0.)
      gamma(1,3,3) = dcmplx(0.,0.)
      gamma(1,3,4) = dcmplx(0.,0.)

      gamma(1,4,1) = dcmplx(-1.,0.) 
      gamma(1,4,2) = dcmplx(0.,0.)
      gamma(1,4,3) = dcmplx(0.,0.)
      gamma(1,4,4) = dcmplx(0.,0.)
***********************************************
*  gamma_y
***********************************************
      gamma(2,1,1) = dcmplx(0.,0.) 
      gamma(2,1,2) = dcmplx(0.,0.)
      gamma(2,1,3) = dcmplx(0.,0.)
      gamma(2,1,4) = dcmplx(0.,-1.)

      gamma(2,2,1) = dcmplx(0.,0.) 
      gamma(2,2,2) = dcmplx(0.,0.)
      gamma(2,2,3) = dcmplx(0.,1.)
      gamma(2,2,4) = dcmplx(0.,0.)

      gamma(2,3,1) = dcmplx(0.,0.) 
      gamma(2,3,2) = dcmplx(0.,1.)
      gamma(2,3,3) = dcmplx(0.,0.)
      gamma(2,3,4) = dcmplx(0.,0.)

      gamma(2,4,1) = dcmplx(0.,-1.) 
      gamma(2,4,2) = dcmplx(0.,0.)
      gamma(2,4,3) = dcmplx(0.,0.)
      gamma(2,4,4) = dcmplx(0.,0.)

***********************************************
*  gamma_z
***********************************************
      gamma(3,1,1) = dcmplx(0.,0.) 
      gamma(3,1,2) = dcmplx(0.,0.)
      gamma(3,1,3) = dcmplx(1.,0.)
      gamma(3,1,4) = dcmplx(0.,0.)

      gamma(3,2,1) = dcmplx(0.,0.) 
      gamma(3,2,2) = dcmplx(0.,0.)
      gamma(3,2,3) = dcmplx(0.,0.)
      gamma(3,2,4) = dcmplx(-1.,0.)

      gamma(3,3,1) = dcmplx(-1.,0.) 
      gamma(3,3,2) = dcmplx(0.,0.)
      gamma(3,3,3) = dcmplx(0.,0.)
      gamma(3,3,4) = dcmplx(0.,0.)

      gamma(3,4,1) = dcmplx(0.,0.) 
      gamma(3,4,2) = dcmplx(1.,0.)
      gamma(3,4,3) = dcmplx(0.,0.)
      gamma(3,4,4) = dcmplx(0.,0.)


***********************************************
*  gamma_5
***********************************************
      gamma5(1,1) = dcmplx(0.,0.) 
      gamma5(1,2) = dcmplx(0.,0.)
      gamma5(1,3) = dcmplx(1.,0.)
      gamma5(1,4) = dcmplx(0.,0.)

      gamma5(2,1) = dcmplx(0.,0.) 
      gamma5(2,2) = dcmplx(0.,0.)
      gamma5(2,3) = dcmplx(0.,0.)
      gamma5(2,4) = dcmplx(1.,0.)

      gamma5(3,1) = dcmplx(1.,0.) 
      gamma5(3,2) = dcmplx(0.,0.)
      gamma5(3,3) = dcmplx(0.,0.)
      gamma5(3,4) = dcmplx(0.,0.)

      gamma5(4,1) = dcmplx(0.,0.) 
      gamma5(4,2) = dcmplx(1.,0.)
      gamma5(4,3) = dcmplx(0.,0.)
      gamma5(4,4) = dcmplx(0.,0.)
*********************************************
*   unitm
*********************************************
      unitm(1,1) = dcmplx(1.,0.) 
      unitm(1,2) = dcmplx(0.,0.)
      unitm(1,3) = dcmplx(0.,0.)
      unitm(1,4) = dcmplx(0.,0.)

      unitm(2,1) = dcmplx(0.,0.) 
      unitm(2,2) = dcmplx(1.,0.)
      unitm(2,3) = dcmplx(0.,0.)
      unitm(2,4) = dcmplx(0.,0.)

      unitm(3,1) = dcmplx(0.,0.) 
      unitm(3,2) = dcmplx(0.,0.)
      unitm(3,3) = dcmplx(1.,0.)
      unitm(3,4) = dcmplx(0.,0.)

      unitm(4,1) = dcmplx(0.,0.) 
      unitm(4,2) = dcmplx(0.,0.)
      unitm(4,3) = dcmplx(0.,0.)
      unitm(4,4) = dcmplx(1.,0.)
*********************************************
*   gmetr
*********************************************
      gmetr(0,0) = dcmplx(1.,0.) 
      gmetr(0,1) = dcmplx(0.,0.)
      gmetr(0,2) = dcmplx(0.,0.)
      gmetr(0,3) = dcmplx(0.,0.)

      gmetr(1,0) = dcmplx(0.,0.) 
      gmetr(1,1) = dcmplx(-1.,0.)
      gmetr(1,2) = dcmplx(0.,0.)
      gmetr(1,3) = dcmplx(0.,0.)

      gmetr(2,0) = dcmplx(0.,0.) 
      gmetr(2,1) = dcmplx(0.,0.)
      gmetr(2,2) = dcmplx(-1.,0.)
      gmetr(2,3) = dcmplx(0.,0.)

      gmetr(3,0) = dcmplx(0.,0.) 
      gmetr(3,1) = dcmplx(0.,0.)
      gmetr(3,2) = dcmplx(0.,0.)
      gmetr(3,3) = dcmplx(-1.,0.)


*****************************************************
*     sigma_\mu_\nu
*****************************************************
      fct = dcmplx(0.0, 0.5)
      do mu = 0,3
      do nu = 0,3
      sigma(mu,nu,1:4,1:4) = fct*
     &  (matmul(gamma(mu,1:4,1:4),gamma(nu,1:4,1:4)) - 
     &   matmul(gamma(nu,1:4,1:4),gamma(mu,1:4,1:4)) )
      enddo
      enddo
******************************************************
*    Levi Civita 
******************************************************
      epsl = 0.0

      epsl(0,1,2,3) = 1.0; epsl(0,3,1,2) = 1.0; epsl(0,2,3,1) = 1.0
      epsl(0,2,1,3) =-1.0; epsl(0,1,3,2) =-1.0; epsl(0,3,2,1) =-1.0

 
      epsl(1,2,0,3) = 1.0; epsl(1,3,2,0) = 1.0; epsl(1,0,3,2) = 1.0
      epsl(1,0,2,3) =-1.0; epsl(1,2,3,0) =-1.0; epsl(1,3,0,2) =-1.0

      epsl(2,0,1,3) = 1.0; epsl(2,3,0,1) = 1.0; epsl(2,1,3,0) = 1.0
      epsl(2,1,0,3) =-1.0; epsl(2,0,3,1) =-1.0; epsl(2,3,1,0) =-1.0

      epsl(3,1,0,2) = 1.0; epsl(3,2,1,0) = 1.0; epsl(3,0,2,1) = 1.0; 
      epsl(3,0,1,2) =-1.0; epsl(3,1,2,0) =-1.0; epsl(3,2,0,1) =-1.0; 

*********************************************
*   big sigma  \vec \Sigma
*********************************************

************************************************
*  \Sigma_x
************************************************
      bigsigma(1,1,1) = dcmplx(0.,0.) 
      bigsigma(1,1,2) = dcmplx(1.,0.) 
      bigsigma(1,1,3) = dcmplx(0.,0.) 
      bigsigma(1,1,4) = dcmplx(0.,0.)
 
      bigsigma(1,2,1) = dcmplx(1.,0.) 
      bigsigma(1,2,2) = dcmplx(0.,0.) 
      bigsigma(1,2,3) = dcmplx(0.,0.) 
      bigsigma(1,2,4) = dcmplx(0.,0.) 

      bigsigma(1,3,1) = dcmplx(0.,0.) 
      bigsigma(1,3,2) = dcmplx(0.,0.) 
      bigsigma(1,3,3) = dcmplx(0.,0.) 
      bigsigma(1,3,4) = dcmplx(1.,0.) 

      bigsigma(1,4,1) = dcmplx(0.,0.) 
      bigsigma(1,4,2) = dcmplx(0.,0.) 
      bigsigma(1,4,3) = dcmplx(1.,0.) 
      bigsigma(1,4,4) = dcmplx(0.,0.) 

************************************************
*  \Sigma_y
************************************************
      bigsigma(2,1,1) = dcmplx(0.,0.) 
      bigsigma(2,1,2) = dcmplx(0.,-1.) 
      bigsigma(2,1,3) = dcmplx(0.,0.) 
      bigsigma(2,1,4) = dcmplx(0.,0.)
 
      bigsigma(2,2,1) = dcmplx(0.,1.) 
      bigsigma(2,2,2) = dcmplx(0.,0.) 
      bigsigma(2,2,3) = dcmplx(0.,0.) 
      bigsigma(2,2,4) = dcmplx(0.,0.) 

      bigsigma(2,3,1) = dcmplx(0.,0.) 
      bigsigma(2,3,2) = dcmplx(0.,0.) 
      bigsigma(2,3,3) = dcmplx(0.,0.) 
      bigsigma(2,3,4) = dcmplx(0.,-1.) 

      bigsigma(2,4,1) = dcmplx(0.,0.) 
      bigsigma(2,4,2) = dcmplx(0.,0.) 
      bigsigma(2,4,3) = dcmplx(0.,1.) 
      bigsigma(2,4,4) = dcmplx(0.,0.) 

************************************************
*  \Sigma_z
************************************************
      bigsigma(3,1,1) = dcmplx(1.,0.) 
      bigsigma(3,1,2) = dcmplx(0.,0.) 
      bigsigma(3,1,3) = dcmplx(0.,0.) 
      bigsigma(3,1,4) = dcmplx(0.,0.)
 
      bigsigma(3,2,1) = dcmplx(0.,0.) 
      bigsigma(3,2,2) = dcmplx(-1.,0.) 
      bigsigma(3,2,3) = dcmplx(0.,0.) 
      bigsigma(3,2,4) = dcmplx(0.,0.) 

      bigsigma(3,3,1) = dcmplx(0.,0.) 
      bigsigma(3,3,2) = dcmplx(0.,0.) 
      bigsigma(3,3,3) = dcmplx(1.,0.) 
      bigsigma(3,3,4) = dcmplx(0.,0.) 

      bigsigma(3,4,1) = dcmplx(0.,0.) 
      bigsigma(3,4,2) = dcmplx(0.,0.) 
      bigsigma(3,4,3) = dcmplx(0.,0.) 
      bigsigma(3,4,4) = dcmplx(-1.,0.) 


      return
      end
