******************************************************************
* Parameterizations for pp and pn scattering
*
*18-Dec-03
* FIU, Miami
******************************************************************


       function sigma_pp_tot(s)
******************************************************************
*      pp total cross section fit in [mb] (sigma_pp)
******************************************************************
       common/par/pi,pm,pmp,pmn,tm,eb  ! pm -> mass of the proton
       dimension B(10)
       ep = (s - 2.0*pm**2)/(2.0*pm)
       p = sqrt(ep**2-pm**2)
       pcm = pm*p/sqrt(s)              ! in GeV
       pcm = pcm/sqrt(0.389385)        ! in 1/sqrt(mb)
       if(p.lt.3.68) then
        call Bystricky_dat(p,0.0,0,B)!using SAID's analysis
        stot = 4.*pi/pcm *  ( B(6)+B(7) )/2.
       elseif ( p.le.5.87) then
        x1 =  3.68
        y1 =  41.39135
        x2 =  3.9
        y2 =  41.19135
        x3 =  5.870000                       
        y3 =  40.42657 
        stot =  cubic_spline(p,x1,x2,x3,y1,y2,y3)
       elseif ( p.le.12.) then
        x1 =  5.87
        y1 =  40.42657
        x2 =  9.
        y2 =  40.19135
        x3 =  12.00000                      
        y3 =  40.01630
        stot =  cubic_spline(p,x1,x2,x3,y1,y2,y3)
       else
        stot = 48.0 + 0.522*alog(p)**2 - 4.51*alog(p)
       endif
       sigma_pp_tot = stot
       return
       end



       function sigma_pn_tot(s)
***************************************************************************
*      pn total cross section fit in [mb]  (sigma_pn)
***************************************************************************
       common/par/pi,pm,pmp,pmn,tm,eb ! pm -> mass of the proton
       dimension B(10)
       ep = (s - 2.0*pm**2)/(2.0*pm)
       p = sqrt(ep**2-pm**2)
       pcm = pm*p/sqrt(s)             ! in GeV
       pcm = pcm/sqrt(0.389385)       ! in 1/sqrt(mb)
       if(p.lt.1.4)then
        call Bystricky_dat(p,0.0,1,B)!using SAID's analysis
        stot = 4.*pi/pcm *  ( B(6)+B(7) )/2.
       elseif(p.le.3.1) then
        x1 =  1.4
        y1 =  38.42728
        x2 =  2.07800
        y2 =  41.905
        x3 =  3.1                     
        y3 =  43.13159
        stot =  cubic_spline(p,x1,x2,x3,y1,y2,y3)
       else
        stot = 47.3 + 0.513*alog(p)**2 - 4.27*alog(p)
       endif
       sigma_pn_tot = stot
       return
       end



       function a_pp(s)
****************************************************************************
*      pp Re/Im amplidtudes ratio  parameter (alpha_pp)
****************************************************************************
       common/par/pi,pm,pmp,pmn,tm,eb  ! pm -> mass of the proton
       dimension B(10)
       ep = (s - 2.0*pm**2)/(2.0*pm)
       p = sqrt(ep**2-pm**2)
       a_pp = 0.0
       if(p.lt.3.68) then
        call Bystricky_dat(p,0.0,0,B)!using SAID's analysis
        a_pp = ( B(1)+B(2) )/( B(6)+B(7) )
       return
       elseif(p.ge.3.68.and.p.lt.5.) then
        x1 =  3.68
        y1 =  -0.3316863
        x2 =  4.34
        y2 =  -0.352568200
        x3 =  5.0
        y3 =  -0.3734501
        a_pp =  cubic_spline(p,x1,x2,x3,y1,y2,y3)
       return
       elseif(p.ge.5..and.p.lt.101.) then
        a_pp=-0.43377 +0.13158E-01*p -0.22820E-03*p**2 
     &       +0.19088E-05*p**3 -0.59601E-08*p**4
       return
       endif
       end



       function a_pn(s)
****************************************************************************
*      pn Re/Im amplidtudes ratio  parameter (alpha_pn)
****************************************************************************
       common/par/pi,pm,pmp,pmn,tm,eb  ! pm -> mass of the proton
       dimension B(10)
       ep = (s - 2.0*pm**2)/(2.0*pm)
       p = sqrt(ep**2-pm**2)
       a_pn = 0.0
       if(p.lt.1.4) then
        call Bystricky_dat(p,0.0,1,B)!using SAID's analysis
        a_pn = ( B(1)+B(2))/( B(6)+B(7))
       return
       elseif(p.ge.1.4.and.p.lt.3.) then
        x1 =  1.4000
        y1 =  -0.2845016
        x2 =  1.9
        y2 =  -0.43915290
        x3 =  3.0
        y3 =  -0.4938042
        a_pn =  cubic_spline(p,x1,x2,x3,y1,y2,y3)
       return
       elseif(p.ge.3..and.p.lt.101.) then
        a_pn=-0.56207 +0.24223E-01*p -0.50362E-03*p**2 
     &       +0.48408E-05*p**3 -0.17331E-07*p**4
       endif
       return
       end



       function bp(s)
***************************************************************************
*      pp Slope parameter (Bpp)
***************************************************************************
       common/par/pi,pm,pmp,pmn,tm,eb ! pm -> mass of the proton
       ep = (s - 2.0*pm**2)/(2.0*pm)
       p = sqrt(ep**2-pm**2)
       if(p.lt.0.926)then
        x1=  0.       
        y1=  0.              
        x2=  0.745
        y2=  0.6
        x3=  0.926  
        y3=  0.39
        bp = cubic_spline(p,x1,x2,x3,y1,y2,y3)
       return
       elseif(p.lt.1.2)then
        x1=  0.9300001       
        y1=  0.39           
        x2=  1.15
        y2=  2.0
        x3=  1.2
        y3=  3.177635
        bp = cubic_spline(p,x1,x2,x3,y1,y2,y3)
       return
       elseif(p.lt.1.7)then
        bp =  55.861-125.66*p+95.365*p**2-22.695*p**3 
       return
       elseif(p.le.3.0)then
        x1 =  1.7
        y1 =  6.343315
        x2 =  2.4
        y2 =  7.5
        x3 =  3.0
        y3 =  7.903550
        bp = cubic_spline(p,x1,x2,x3,y1,y2,y3)
       return
       else
        bp  = 8.22  + 1.10*log(p/4.)
       endif
       return
       end



       function bn(s)
***************************************************************************
*      pn Slope parameter (Bpn)
***************************************************************************
       common/par/pi,pm,pmp,pmn,tm,eb ! pm -> mass of the proton
       dimension B1(10),B2(10)
       ep = (s - 2.0*pm**2)/(2.0*pm)
       p = sqrt(ep**2-pm**2)
       if(p.lt.0.8)then
        x1=  0.       
        y1=  0.              
        x2=  0.545
        y2=  1.0 
        x3=  0.8000000       
        y3=  2.232239 
        bn =  cubic_spline(p,x1,x2,x3,y1,y2,y3)
       return  
       elseif(p.lt.1.74)then  
        bn = -0.41579 + 1.8333*p + 2.6484*p**2  -1.0031*p**3              
       return  
       elseif(p.le.3.0)then
        x1 =  1.740000   
        y1 =  5.508093 
        x2 =  2.4
        y2 =  7.202996 
        x3 =  3.0
        y3 =  7.903550 
        bn =  cubic_spline(p,x1,x2,x3,y1,y2,y3)
       return  
       else  
        bn  = 8.22  + 1.10*alog(p/4.)
       endif  
       return 
       end





        

       function cubic_spline(p,x1,x2,x3,y1,y2,y3)
****************************************************************************
*      Approximates function with the 3-nodes cubic spline
****************************************************************************
       dyx2 = (y3-y2)/(x3-x2)
       dyx1 = (y2-y1)/(x2-x1) 
       gamma2 = 6.*( dyx2 - dyx1 )
       yyy2 = gamma2/2./(x3-x1)
       yy1  = dyx1 - yyy2*(x2-x1)/6.
       yy2  = dyx2 - yyy2*(x3-x2)/3.
       C1   = y1+yy1*(p-x1)+yyy2*(p-x1)**3/(x2-x1)/6. 
       C2   = y2+yy2*(p-x2)+yyy2*(p-x2)**2/2.
     &                     -yyy2*(p-x2)**3/(x3-x2)/6.
       if(p.le.x2) cubic_spline = C1
       if(p.gt.x2) cubic_spline = C2
       return 
       end
