        subroutine
     & wf_he3_nb(mt1,ms1,pv1,mt2,ms2,pv2,mt3,ms3,kj,wfre,wfim,iflag)
***********************************************************************
*
* mt1,mt2,mt3 - isospin projections of 1,2,3 nucleons -1-neutron,1-proton
* ms1,ms2,ms3 - spin projection of 1,2,3 nucleon -1-> -1/2, 1-> 1/2
* kj - spin projection of he3 -1 -> -1/2, 1-> 1/2
* pv1,pv2 - three vectors of nucleon 1 and 2 in GeV/c
* wfre,wfim - re and im part of the wave function in GeV-3
* iflag = 1,2,0, 1 - read the wave function data, 2 -reset the value of kj
*         0 - calculate
*
* Normalization
* \int |psi|^2d^3p1d^3p2   = 1 averaged by kj
* \int |psi_p|^2d^3p1d^3p2 = 2/3
* \int |psi_n|^2d^3p1d^3p2 = 1/3
*
* 21-February-2003, Miami
* 
***********************************************************************
       use getwavemod
       dimension pv1(3),pv2(3)
       CHARACTER(LEN=80) :: wavein
       COMPLEX wave
       wfre = 0.0
       wfim = 0.0
       fmg = 0.197326
       mbj =  kj
       if(iflag.eq.1.)then
*        WAVEIN = 'he_3.cdbonn.form'
*       WAVEIN = 'he_3.cdbonn+tml4784.form' ! not provided
*       WAVEIN = 'he_3.av18+tml5156.form'  ! not provided
*        WAVEIN = 'he_3.av18+urb-ix.form'
         WAVEIN = 'he_3.av18.form'
       CALL readwave(WAVEIN,1,.false.)
*21apr       write(6,*)wavein
       return
       elseif(iflag.eq.2)then          
       CALL mbjreset(mbj,.false.)
        wfre = 0.0
        wfim = 0.0      
       return
       endif


      p1x = pv1(1)/fmg
      p1y = pv1(2)/fmg
      p1z = pv1(3)/fmg

      p2x = pv2(1)/fmg
      p2y = pv2(2)/fmg
      p2z = pv2(3)/fmg

      p3x = -(p1x+p2x)
      p3y = -(p1y+p2y)
      p3z = -(p1z+p2z)

      wave=getwave(p1x,p1y,p1z,ms1,mt1,p2x,p2y,p2z,ms2,mt2,ms3,mt3)
     X    +getwave(p2x,p2y,p2z,ms2,mt2,p3x,p3y,p3z,ms3,mt3,ms1,mt1)
     X    +getwave(p3x,p3y,p3z,ms3,mt3,p1x,p1y,p1z,ms1,mt1,ms2,mt2)


      wfre = real(wave)/fmg**3
      wfim = aimag(wave)/fmg**3
*      write(6,*)"wave",ms1,mt1,ms2,mt2,pv1,pv2,wave
      return
      end
      
       subroutine
     & wf_h3_nb(mt10,ms1,pv1,mt20,ms2,pv2,mt30,ms3,kj,wfre,wfim,iflag)
***********************************************************************
*
* H3 wave function built as a mirror reflection of he3
* mt10,mt20,mt30 - isospin projections of 1,2,3 nucleons -1-neutron,1-proton
* ms1,ms2,ms3 - spin projection of 1,2,3 nucleon -1-> -1/2, 1-> 1/2
* kj - spin projection of he3 -1 -> -1/2, 1-> 1/2
* pv1,pv2 - three vectors of nucleon 1 and 2 in GeV/c
* wfre,wfim - re and im part of the wave function in GeV-3
* iflag = 1,2,0, 1 - read the wave function data, 2 -reset the value of kj
*         0 - calculate
*
* Normalization
* \int |psi|^2d^3p1d^3p2   = 1 averaged by kj
* \int |psi_p|^2d^3p1d^3p2 = 1/3
* \int |psi_n|^2d^3p1d^3p2 = 2/3
*
* 11-April-2013, Miami
* 
***********************************************************************
       use getwavemod
       dimension pv1(3),pv2(3)
       CHARACTER(LEN=80) :: wavein
       COMPLEX wave
       wfre = 0.0
       wfim = 0.0
       fmg = 0.197326
       mbj =  kj
       if(iflag.eq.1.)then
*        WAVEIN = 'he_3.cdbonn.form'
*       WAVEIN = 'he_3.cdbonn+tml4784.form'
*       WAVEIN = 'he_3.av18+tml5156.form'
*        WAVEIN = 'he_3.av18+urb-ix.form'
*         WAVEIN = 'he_3.av18.form'
       CALL readwave(WAVEIN,1,.false.)
*21apr       write(6,*)wavein
       return
       elseif(iflag.eq.2)then          
       CALL mbjreset(mbj,.false.)
        wfre = 0.0
        wfim = 0.0      
       return
       endif

       mt1 = -mt10
       mt2 = -mt20
       mt3 = -mt30

      p1x = pv1(1)/fmg
      p1y = pv1(2)/fmg
      p1z = pv1(3)/fmg

      p2x = pv2(1)/fmg
      p2y = pv2(2)/fmg
      p2z = pv2(3)/fmg

      p3x = -(p1x+p2x)
      p3y = -(p1y+p2y)
      p3z = -(p1z+p2z)

      wave=getwave(p1x,p1y,p1z,ms1,mt1,p2x,p2y,p2z,ms2,mt2,ms3,mt3)
     X    +getwave(p2x,p2y,p2z,ms2,mt2,p3x,p3y,p3z,ms3,mt3,ms1,mt1)
     X    +getwave(p3x,p3y,p3z,ms3,mt3,p1x,p1y,p1z,ms1,mt1,ms2,mt2)


      wfre = real(wave)/fmg**3
      wfim = aimag(wave)/fmg**3
*      write(6,*)"wave",mt1,mt2,mt3,wave
      return
      end
      
